/*
 * Created on Feb 3, 2004
 */
package org.lucci.madhoc.broadcast;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;

import org.lucci.madhoc.broadcast.gui.BroadcastAircraftView;
import org.lucci.madhoc.broadcast.impl.research.dfcn.DFCN;
import org.lucci.madhoc.broadcast.impl.research.dfcn.test.FreeProactiveDFCN;
import org.lucci.madhoc.broadcast.impl.research.dfcn.test.PurelyProactiveDFCN;
import org.lucci.madhoc.broadcast.impl.research.dfcn.test.PurelyReactiveDFCN;
import org.lucci.madhoc.broadcast.impl.standard.AHBP_EX;
import org.lucci.madhoc.broadcast.impl.standard.MultiPointRelaying;
import org.lucci.madhoc.broadcast.measure.AverageBroadcastingEfficiency;
import org.lucci.madhoc.broadcast.measure.AverageCoverageMeasure;
import org.lucci.madhoc.broadcast.measure.AverageEmissionEfficiencyMeasure;
import org.lucci.madhoc.broadcast.measure.AverageHostCountMeasure;
import org.lucci.madhoc.broadcast.measure.AverageMemoryUsageMeasure;
import org.lucci.madhoc.broadcast.measure.AverageNumberOfEmissionMeasure;
import org.lucci.madhoc.broadcast.measure.AverageNumberOfRedundantReceptionsMeasure;
import org.lucci.madhoc.broadcast.measure.EmissionDistributionMeasure;
import org.lucci.madhoc.broadcast.measure.NumberOfMessages;
import org.lucci.madhoc.broadcast.measure.NumberOfMessagesSentByProactivity;
import org.lucci.madhoc.broadcast.measure.NumberOfMessagesSentByReactivity;
import org.lucci.madhoc.broadcast.measure.RatioOfRebroadcastingStationsMeasure;
import org.lucci.madhoc.messaging.TransferableObject;
import org.lucci.madhoc.network.Application;
import org.lucci.madhoc.network.Station;
import org.lucci.madhoc.simulation.Configurable;
import org.lucci.madhoc.simulation.Monitor;


public class BroadcastingMonitor extends Monitor implements Configurable
{
	private int numberOfMessageToCreate = 1;

	private double messageLifeTime = 10;
	private int broadcastMaxHop = 10;

	
	public final static int ONE_TO_ALL = 1;
	public final static int ALL_TO_ALL = 2;
	public final static int ONE_TENTH_TO_ALL = 3;
	public final static int TEN_ALL = 4;
	private int mode = ONE_TO_ALL;

	// the ads being broadcasted
	private Collection objects = new Vector(); 
	
	
	public BroadcastingMonitor()
	{
		addMeasure(new AverageNumberOfEmissionMeasure());
		addMeasure(new AverageNumberOfRedundantReceptionsMeasure());
		addMeasure(new AverageHostCountMeasure());
		addMeasure(new AverageCoverageMeasure());
		addMeasure(new AverageEmissionEfficiencyMeasure());
		addMeasure(new AverageBroadcastingEfficiency());
		addMeasure(new RatioOfRebroadcastingStationsMeasure());
		addMeasure(new AverageMemoryUsageMeasure());
		addMeasure(new NumberOfMessages());
		addMeasure(new NumberOfMessagesSentByReactivity());
		addMeasure(new NumberOfMessagesSentByProactivity());

		addMeasure(new EmissionDistributionMeasure());
	}

	/* (non-Javadoc)
	 * @see org.lucci.madhoc.config.Configurable#configure()
	 */
	public void configure() throws Throwable
	{
		super.configure();
		setMessageLifeTime(getNetwork().getSimulation().getConfiguration().getDouble("broadcast_message_lifetime"));
		setNumberOfMessageToCreate(getNetwork().getSimulation().getConfiguration().getInteger("broadcast_message_count"));
		setBroadcastMaxHop(getNetwork().getSimulation().getConfiguration().getInteger("broadcast_max_hop"));
	}

	public double getAverageMemoryRequired(Collection<Station> computers)
	{
		double sz = 0;
		
		for (Station s : computers)
		{
            BroadcastingProtocol p = (BroadcastingProtocol) s.getApplicationMap().getValue(BroadcastingProtocol.class);
			sz += p.getMemoryRequired();
		}
		
		return sz / getAverageNumberOfHosts(computers);
	}

	
	public int getMode()
	{
		return mode;
	}
	public void setMode(int mode)
	{
		this.mode = mode;
	}


	

	public Collection getApplicationsHosting(Collection computers, TransferableObject object)
	{
		Collection hostingApps = new Vector();
		Iterator stationApplications = findApplications(computers).iterator();
			
		while (stationApplications.hasNext())
		{
			BroadcastingProtocol app = (BroadcastingProtocol) stationApplications.next();

			if (app.getLocalInfo(object) != null)
			{
				hostingApps.add(app);
			}
		}

		return hostingApps;
	}


	
	public double getHostingRate(Collection computers, TransferableObject object)
	{
		return (double) getApplicationsHosting(computers, object).size() / (double) findApplications(computers).size();
	}

	public double getAverageNumberOfHosts(Collection computers)
	{
		if (this.objects.isEmpty())
			throw new IllegalStateException("no broadcast messages");

		double sum = 0;
		Iterator adIterator = this.objects.iterator();
		
		while (adIterator.hasNext())
		{
			TransferableObject object = (TransferableObject) adIterator.next();
			sum += getApplicationsHosting(computers, object).size();
		}

		return sum / this.objects.size();
	}

	
	public double getAverageCoverageRate(Collection computers)
	{
		return getAverageNumberOfHosts(computers) / computers.size();
	}

	public double getAverageNumberOfEmissions(Collection computers)
	{
		double totalNumberOfEmissions = 0;
		Iterator appIterator = findApplications(computers).iterator();
		
		while (appIterator.hasNext())
		{
			BroadcastingProtocol p = (BroadcastingProtocol) appIterator.next();
			Iterator infoIterator = p.getHostedObjectsMap().values().iterator();
			
			while (infoIterator.hasNext())
			{
				LocalInfo info = (LocalInfo) infoIterator.next();
				totalNumberOfEmissions += info.emissionCount;
			}
		}
		
		return totalNumberOfEmissions / objects.size();
	}


	public double getAverageNumberOfRedundantReceptions(Collection computers)
	{
		double totalNumberOfReceptions = 0;
		Iterator appIterator = findApplications(computers).iterator();
		
		while (appIterator.hasNext())
		{
			BroadcastingProtocol p = (BroadcastingProtocol) appIterator.next();
			Iterator infoIterator = p.getHostedObjectsMap().values().iterator();
			
			while (infoIterator.hasNext())
			{
				LocalInfo info = (LocalInfo) infoIterator.next();
				totalNumberOfReceptions += info.redundantReceptionCount;
			}
		}
		
		return totalNumberOfReceptions / objects.size();
	}

	public double getEmissionEfficiency(Collection computers)
	{
		double hostCount = getAverageNumberOfHosts(computers);
		double emissionCount = getAverageNumberOfEmissions(computers);

		if (emissionCount > 0)
		{
			return hostCount / emissionCount;
		}
		else
		{
			throw new IllegalStateException("no emission have been carried out till now");
		}
	}

	/**
	 * If at least 1 station allows multiple rebroadcasting, then we consider it is allowed on the network.
	 * @return
	 */
	public boolean isMultipleRebroadcastingAllowed()
	{
		Collection applications = findApplications(getNetwork().getStations());

		if (applications.isEmpty())
		{
			throw new IllegalStateException("no station support a broadcasting protocol");
		}
		else
		{
			Iterator protocolIterator = applications.iterator();
			
			// ensure that all broadcasting protocol on the network agree with that
			while (protocolIterator.hasNext())
			{
				BroadcastingProtocol protocol = (BroadcastingProtocol) protocolIterator.next();
				
				if (protocol.isMultipleRebroadcastAllowed())
				{
					return true;
				}
			}
			
			return false;
		}
	}
	
	public int getNumberOfRebroadcastingStations(Collection computers)
	{
		if (getMode() == ONE_TO_ALL)
		{
			if (isMultipleRebroadcastingAllowed())
			{
				throw new IllegalStateException("cannot defined the number of rebroadcasting stations since multiple rebroadcasting is allowed");
			}
			else
			{
				int numberOfStations = 0;
				Iterator appIterator = findApplications(computers).iterator();
				
				while (appIterator.hasNext())
				{
					BroadcastingProtocol p = (BroadcastingProtocol) appIterator.next();
					
					if (p.getAverageNumberOfEmissions() > 0)
					{
						++numberOfStations;
					}
				}
				
				return numberOfStations;
			}
		}
		else
		{
			throw new IllegalStateException("cannot defined the number of rebroadcasting stations since you're not carrying out a one2one broadcast");
		}
	}
	
	public double getRatioOfRebroadcastingStations(Collection computers)
	{
		return getNumberOfRebroadcastingStations(computers) / getAverageNumberOfHosts(computers);
	}

	
	
	/* (non-Javadoc)
	 * @see org.lucci.madhoc.simulation.SimulationApplication#getName()
	 */
	public String getName()
	{
		Collection protocolClasses = new HashSet();
		Iterator i = findApplications(getNetwork().getStations()).iterator();
		
		while (i.hasNext())
		{
			Application stationApplication = (Application) i.next();
			protocolClasses.add(stationApplication.getClass());
		}
		
		if (protocolClasses.size() == 0)
		{
			return "broadcasting";
		}
		else if (protocolClasses.size() == 1)
		{
			return "broadcasting" + " (" + ((Application) findApplications(getNetwork().getStations()).iterator().next()).getName() + ')';
		}
		else
		{
			return "broadcasting (mixed)";
		}
	}

	/* (non-Javadoc)
	 * @see org.lucci.madhoc.simulation.SimulationApplication#getPublicName()
	 */
	public String getFriendlyName()
	{
		return getName();
	}
	

	/* (non-Javadoc)
	 * @see org.lucci.madhoc.simulation.SimulationApplication#resetIterationScopedValues()
	 */
	public void resetIterationScopedValues()
	{
	}

	public Collection getObjects()
	{
		return objects;
	}
	
	public Collection getAvailableBroadcastingProtocols()
	{
		Collection c = new Vector();
		c.add(org.lucci.madhoc.broadcast.impl.standard.SimpleFlooding.class);
		c.add(org.lucci.madhoc.broadcast.impl.standard.ProbabilisticScheme.class);
		c.add(org.lucci.madhoc.broadcast.impl.standard.FloodingWithSelfPruning.class);
		c.add(org.lucci.madhoc.broadcast.impl.standard.LocationBasedScheme.class);
		c.add(org.lucci.madhoc.broadcast.impl.standard.CounterBasedScheme.class);
		c.add(org.lucci.madhoc.broadcast.impl.standard.SBA.class);
		c.add(MultiPointRelaying.class);
		c.add(AHBP_EX.class);

		c.add(DFCN.class);
		c.add(org.lucci.madhoc.broadcast.impl.research.dfcn.test.FreeDFCN.class);
		c.add(PurelyProactiveDFCN.class);
		c.add(PurelyReactiveDFCN.class);
		c.add(FreeProactiveDFCN.class);
		return c;
	}

	
	/* (non-Javadoc)
	 * @see org.lucci.madhoc.simulation.SimulationApplication#getSimulationApplicationViewClasses()
	 */
	public Collection getMonitorViewClasses()
	{
		Vector v = new Vector();
		v.add(BroadcastAircraftView.class);
		return v;
	}
	public int getNumberOfMessageToCreate()
	{
		return numberOfMessageToCreate;
	}
	public void setNumberOfMessageToCreate(int numberOfMessageToCreate)
	{
		if (numberOfMessageToCreate < 0)
			throw new IllegalArgumentException("cannot create a negative number of messages");

		this.numberOfMessageToCreate = numberOfMessageToCreate;
	}
	

	public double getMessageLifeTime()
	{
		return messageLifeTime;
	}
	public void setMessageLifeTime(double messageLifeTime)
	{
		this.messageLifeTime = messageLifeTime;
	}

	public int getBroadcastMaxHop()
	{
		return broadcastMaxHop;
	}

	public void setBroadcastMaxHop(int broadcastMaxHop)
	{
		if (broadcastMaxHop < 0)
			throw new IllegalArgumentException("should be >= 0");
	
		this.broadcastMaxHop = broadcastMaxHop;
	}

}
