package org.lucci.madhoc.broadcast;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Vector;

import org.lucci.madhoc.network.Station;
import org.lucci.madhoc.network.util.Geometry;
import org.lucci.madhoc.network.util.Graph;
import org.lucci.madhoc.simulation.MonitorInitializer;

/*
 * Created on May 31, 2005
 */

/**
 * @author luc.hogie
 */
public class BroadcastingMonitorInitializer extends MonitorInitializer
{
	public void initialize()
	{
		BroadcastingMonitor broadcastingMonitor = (BroadcastingMonitor) getMonitor();
		Collection<Station> initialStations = getInitialStations();
		
		for(Station station : initialStations)
		{
			BroadcastingProtocol protocol = (BroadcastingProtocol) station.getApplicationMap().getValue(BroadcastingProtocol.class);
	
			for (int i = 0; i < broadcastingMonitor.getNumberOfMessageToCreate(); ++i)
			{
				Ad ad = new Ad();
				ad.setExpirationDate(broadcastingMonitor.getNetwork().getSimulation().getSimulatedTime() + ((BroadcastingMonitor) getMonitor()).getMessageLifeTime());
				ad.setCreatorStation(protocol.getComputer());
				ad.setType(broadcastingMonitor.getNetwork().getSimulation().getRandomNumberGenerator().getRandom().nextDouble() < 0.5 ? Ad.SELL : Ad.SEARCH);
				ad.setBrand("bohbrand");
				ad.setModel("bohmodel");
				ad.setColor(broadcastingMonitor.getColor());
				protocol.addObject(ad);
				broadcastingMonitor.getObjects().add(ad);
			}
		}
	}

	protected Collection<Station> getInitialStations()
	{
		int mode = ((BroadcastingMonitor) getMonitor()).getMode();

        if (mode == BroadcastingMonitor.ONE_TO_ALL)
		{
			List<Station> stationList = Geometry.getStationsOrderByTheirDistanceToTheBarycenter(getMonitor().getNetwork());
			double averageDegree = Graph.getAverageDegree(getMonitor().getNetwork());

			// choose a well-connected station
			for (int i = 0; i < stationList.size(); ++i)
			{
				Station randomStation = stationList.get(i);
				
				if (randomStation.getNetworkingUnit().getNeighborhood().size() > averageDegree)
				{
					return Arrays.asList(new Station[] {randomStation});
				}
			}
			
			return Arrays.asList(new Station[] {stationList.get(0)});
		}
		else if (mode == BroadcastingMonitor.ALL_TO_ALL)
		{
			return new Vector<Station>(getMonitor().getNetwork().getStations());
		}
		else if (mode == BroadcastingMonitor.ONE_TENTH_TO_ALL)
		{
			int oneTenth = getMonitor().getNetwork().getStations().size() / 10;
			Collection<Station> stations = new HashSet<Station>();
			
			for (int i = 0; i < oneTenth; ++i)
			{
				Random random = getMonitor().getNetwork().getSimulation().getRandomNumberGenerator().getRandom();
				stations.add((Station) org.lucci.util.Collections.getRandomObject(getMonitor().getNetwork().getStations(), random));
			}
			
			return stations;
		}
		else
		{
			throw new IllegalStateException();
		}
	}

	/* (non-Javadoc)
	 * @see org.lucci.madhoc.config.Configurable#configure()
	 */
	public void configure() throws Throwable
	{
	}
	
}
