package org.lucci.madhoc.broadcast;

import org.lucci.madhoc.messaging.Message;
import org.lucci.madhoc.messaging.TransferableObject;
import org.lucci.math.Utilities;

/*
 * Created on Aug 20, 2004
 */

/**
 * @author luc.hogie
 */
public abstract class RandomDelayBasedBroadcastingProtocol extends BroadcastingProtocol
{
	private double minimumDelay = 0;
	private double maximumDelay = Integer.MAX_VALUE;


	public void configure()
		throws Throwable
	{
		double[] pauseInterval = getMonitor().getNetwork().getSimulation().getConfiguration().getInterval("broadcast_rad_interval");
		setMinimumDelay(pauseInterval[0]);
		setMaximumDelay(pauseInterval[1]);
	}


	public boolean isBroadcastWorth(TransferableObject object)
	{
		LocalInfo info = getLocalInfo(object);

		// it is now time to consider sending the message
		if (getSimulatedTime() >= info.retransmissionDate)
		{
			// the decision has not been taken
			// it could have been taken, for instance, at reception time
			if (info.decision == -1)
			{
				// then take it!!!
				takeDecision(object);
			}

			return info.decision == 1;
		}
		else
		{
			return false;
		}
	}

	
	/**
	 * @param ad
	 */
	protected abstract void takeDecision(TransferableObject ad);




	protected void messageJustReceived(Message msg)
	{
		super.messageJustReceived(msg);
		LocalInfo info = getLocalInfo(msg.getContent());
		
		if (info.retransmissionDate == -1)
		{
			double rad = Utilities.getRandomBetween(getMinimumDelay(), getMaximumDelay(), getMonitor().getNetwork().getSimulation().getRandomNumberGenerator().getRandom());
			info.retransmissionDate = getSimulatedTime() + rad;
		}
	}


	public double getMaximumDelay()
	{
		return maximumDelay;
	}

	public void setMaximumDelay(double d)
	{
		if (d < 0)
			throw new IllegalArgumentException("max rad cannot be negative");

		if (d < getMinimumDelay())
			throw new IllegalArgumentException("the maximum you specifies is lower than the minimum delay");

		maximumDelay = d;
	}

	public double getMinimumDelay()
	{
		return minimumDelay;
	}

	public void setMinimumDelay(double d)
	{
		if (d < 0)
			throw new IllegalArgumentException("min rad cannot be negative");

		if (d > getMaximumDelay())
			throw new IllegalArgumentException("the minimum you specifies is greater than the maximum delay");

		this.minimumDelay = d;
	}
}
