package org.lucci.madhoc.broadcast;

import java.util.Collection;

import org.lucci.madhoc.broadcast.measure.AverageCoverageMeasure;
import org.lucci.madhoc.simulation.Monitor;
import org.lucci.madhoc.simulation.TerminationCondition;
import org.lucci.madhoc.simulation.measure.MeasureHistory;
import org.lucci.madhoc.simulation.projection.IdentityProjection;
import org.lucci.madhoc.simulation.projection.Projection;
import org.lucci.math.Utilities;

/*
 * Created on Apr 26, 2005
 */

/**
 * @author luc.hogie
 */
public class StableCoverageTerminationCondition extends TerminationCondition
{
	private double observationTimeInterval = 10;

	/* (non-Javadoc)
	 * @see org.lucci.madhoc.simulation.SimulationTerminationCondition#simulationHasCompleted(org.lucci.madhoc.simulation.Simulation)
	 */
	public boolean applicationHasCompleted(Monitor app)
	{
		BroadcastingMonitor broadcastingApp = (BroadcastingMonitor) app;
		double simulatedTime = broadcastingApp.getNetwork().getSimulation().getSimulatedTime();

		if (simulatedTime > getObservationTimeInterval())
		{
			int iteration = broadcastingApp.getNetwork().getSimulation().getIteration();
			double startTime = simulatedTime - getObservationTimeInterval();
			double resolution = broadcastingApp.getNetwork().getSimulation().getResolution();
			int startIteration = (int) (startTime / resolution);
			Projection projection = (Projection) app.getNetwork().getProjectionMap().get(IdentityProjection.class);
			MeasureHistory measures = (MeasureHistory) projection.getMeasureHistoryMap().get(AverageCoverageMeasure.class);
			Collection relevalMeasures = measures.getValues().subList(startIteration, iteration);
//			double sd = Utilities.getStandardDeviation(relevalMeasures);
			double sd = Utilities.getVariationSum(relevalMeasures);
			return sd == 0;
		}
		else
		{
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see org.lucci.madhoc.config.Configurable#configure()
	 */
	public void configure() throws Throwable
	{
		setObservationTimeInterval(Double.valueOf(getMonitor().getNetwork().getSimulation().getConfiguration().getConfigurationValue("observation_time_interval")).doubleValue());
	}

	
	
	public double getObservationTimeInterval()
	{
		return observationTimeInterval;
	}
	public void setObservationTimeInterval(double observationTimeInterval)
	{
		this.observationTimeInterval = observationTimeInterval;
	}
}
