package org.lucci.madhoc.broadcast.batch;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import org.lucci.madhoc.broadcast.BroadcastingMonitor;
import org.lucci.madhoc.broadcast.impl.standard.CounterBasedScheme;
import org.lucci.madhoc.broadcast.measure.AverageEmissionEfficiencyMeasure;
import org.lucci.madhoc.env.mall.MallEnvironment;
import org.lucci.madhoc.network.Network;
import org.lucci.madhoc.network.monitor.NetworkMonitor;
import org.lucci.madhoc.script.Batch;
import org.lucci.madhoc.simulation.MadhocSimulation;
import org.lucci.madhoc.simulation.measure.NumericalSensor;
import org.lucci.math.Utilities;
import org.lucci.math.relation.Function;

/*
 * Created on Jul 2, 2004
 */

/**
 * @author luc.hogie
 */
public class FindTheBestCounterBasedThreshold extends Batch
{
	public static void main(String[] args)
	{
		new FindTheBestCounterBasedThreshold();
	}
	
	public FindTheBestCounterBasedThreshold()
	{
		Collection functions = new Vector();
		
		for (int i = 0; i < 1; ++i)
		{
			println("Running test " + i);
			functions.add(getThresholdPerformanceFunction());
		}
		
		Function avg = Function.getAverageFunction(functions);
		println("result:\n" + avg.getGNUPlotText(avg.getKeys()));
	}

	private Function getThresholdPerformanceFunction()
	{
		Function res = new Function();
 
		for (int threshold = 0; threshold < 20; ++threshold)
		{
			println("\ttesting threshold = " + threshold);
			double d = getThresholdPerformance(threshold);
			res.add(new Double(threshold), new Double(d));
		}
		
		return res;
	}
	
	private double getThresholdPerformance(int threshold)
	{
		Collection c = new Vector();

		for (int i = 0; i < 20; ++i)
		{
			println("\t\ttest " + i);
			MadhocSimulation simulation = testThreshold(threshold);
			Double d = (Double) ((NumericalSensor) simulation.findSensor(AverageEmissionEfficiencyMeasure.class)).getLastValue();
			c.add(d);
		}
		
		return Utilities.getAverage(c);
	}

	
	private MadhocSimulation testThreshold(int threshold)
	{
		// creates the network, which is initially empty
		Network net = new Network();
		
		// define a natural environment for the network
		// this environment will have a great impact on the structure of the network
		// for instance, the city environment will incorporate walls that will prevent the
		// radio waves from propagating
		// here we define a 1 square kilometer (1 million square meter)
		// open area (which could be a field) 
		net.setNetworkEnvironment(new MallEnvironment(250000, 100));

		// creates the stations that will populate the network
		net.createStations(5000);
		
		// the stations have been located randomly on the environment
		// it might be interested to let them run for a while in order
		// to stabilize the network graph
		net.initializeStationLocation();

		// once we consider the location of the station has been initialized
		// we establish the connections that define the network itself
		net.createNewConnections();

		// the network will be handled by a simulation
		MadhocSimulation simulation = new MadhocSimulation();
		simulation.setName("test simulation with threshold=" + threshold);
		simulation.setNetwork(net);

		// the broadcasting application is what we want to run and observe
		BroadcastingMonitor broadcastingApp = new BroadcastingMonitor();


		// it is now time to define the applications that will run on the network
		{
			Collection applications = new Vector();

			{
				// the "core" application is a basic application that aim at extracting some
				// general purpose information on the simulation process
				applications.add(new NetworkMonitor());
			}
			{
				// define the broadcasting strategy we want to carry out
				broadcastingApp.setStationApplicationClass(CounterBasedScheme.class);
				broadcastingApp.setMode(BroadcastingMonitor.ONE_TO_ALL);
				applications.add(broadcastingApp);
			}

			// the simulation knows the application is has to carry out
			// now let deploy them over the network
			simulation.deployApplications(applications);
			
			// tells each station to use the threshold
			{
				Iterator i = broadcastingApp.getStationApplications().iterator();
				
				while (i.hasNext())
				{
					((CounterBasedScheme) i.next()).setThreshold(threshold);
				}
			}
		}

		// at this step the simulation is ready to be used
		// the user can get information on the simulation process via the Simulation.getMeasures() method,
		// via the methods provided by the various objects (Simulation, Network, Station...)
		// or via the discrete event file, depending on his own needs
		BroadcastingMonitor broadcastApp = (BroadcastingMonitor) simulation.getMonitorMap().get(BroadcastingMonitor.class);
		
		while (broadcastApp.getAverageCoverageRate() < 0.4 && simulation.getSimulatedTime() < 1000 * 6)
		{
			// each iteration represents 250ms of simulated time
			// if you want to simulate 1 hour, you then have to execute 4 * 60 * 60 iterations or
			// iterate while Simulation.getSimulatedTimeMillis() < 3600
			simulation.iterate();
		}

		return simulation;
	}

}
