package org.lucci.madhoc.broadcast.batch;

import java.io.File;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.lucci.madhoc.broadcast.BroadcastingMonitor;
import org.lucci.madhoc.broadcast.impl.research.Proba;
import org.lucci.madhoc.broadcast.impl.research.dfcn.DFCN;
import org.lucci.madhoc.broadcast.impl.research.dfcn.FreeDFCN;
import org.lucci.madhoc.broadcast.impl.research.dfcn.test.FreeProactiveDFCN;
import org.lucci.madhoc.broadcast.impl.research.dfcn.test.PurelyProactiveDFCN;
import org.lucci.madhoc.broadcast.impl.research.dfcn.test.PurelyReactiveDFCN;
import org.lucci.madhoc.broadcast.impl.standard.MultiPointRelaying;
import org.lucci.madhoc.broadcast.impl.standard.AHBP_EX;
import org.lucci.madhoc.broadcast.impl.standard.ProbabilisticScheme;
import org.lucci.madhoc.broadcast.measure.AverageBroadcastingEfficiency;
import org.lucci.madhoc.broadcast.measure.AverageCoverageMeasure;
import org.lucci.madhoc.broadcast.measure.AverageEmissionEfficiencyMeasure;
import org.lucci.madhoc.broadcast.measure.AverageNumberOfEmissionMeasure;
import org.lucci.madhoc.broadcast.measure.AverageNumberOfRedundantReceptionsMeasure;
import org.lucci.madhoc.env.mall.MallEnvironment;
import org.lucci.madhoc.env.open_area.OpenAreaEnvironment;
import org.lucci.madhoc.network.Network;
import org.lucci.madhoc.network.monitor.NetworkMonitor;
import org.lucci.madhoc.network.monitor.measure.ConnectionBandwithDistributionMeasure;
import org.lucci.madhoc.network.monitor.measure.DegreeDistributionMeasure;
import org.lucci.madhoc.network.monitor.measure.LoadDistributionMeasure;
import org.lucci.madhoc.network.monitor.measure.NumberOfPartitions;
import org.lucci.madhoc.network.monitor.measure.PartitionDistributionMeasure;
import org.lucci.madhoc.network.monitor.measure.SimulatedTimeMeasure;
import org.lucci.madhoc.network.util.Graph;
import org.lucci.madhoc.script.Batch;
import org.lucci.madhoc.simulation.MadhocSimulation;
import org.lucci.madhoc.simulation.Monitor;
import org.lucci.madhoc.simulation.measure.Sensor;
import org.lucci.madhoc.simulation.measure.NumericalSensor;
import org.lucci.math.Utilities;
import org.lucci.math.relation.DefaultRelation;
import org.lucci.math.relation.Function;
import org.lucci.math.relation.Relation;

/*
 * Created on Aug 18, 2004
 */

/**
 * 
 * For each broadcasting protocol:
 * vary the density of mobile stations in [500, 10000]
 * and measures the hosting rate, emission efficiency
 * each mesure being the average of 10 measures
 * 
 * 
 * 
 * @author luc.hogie
 */
public class TestIfDFCNCanDoFullCoverage extends Batch
{
	private int tests = 3;
//	int[] steps = {10, 50, 100, 300, 500, 750, 1000, 1500, 2000, 3000};
	int[] steps = {500, 1250, 2000};
	
	public static void main(String[] args)
	{
		new TestIfDFCNCanDoFullCoverage();
	}
	
	public TestIfDFCNCanDoFullCoverage()
	{
		test();
	}

	private void test()
	{
		println("Starting tests...");
//		testProtocol(org.lucci.madhoc.broadcast.impl.standard.SimpleFlooding.class);
//		testProtocol(org.lucci.madhoc.broadcast.impl.standard.FloodingWithSelfPruning.class);
//		testProtocol(org.lucci.madhoc.broadcast.impl.standard.SBA.class);
//		testProtocol(AHBP.class);
//		testProtocol(ProbabilisticScheme.class);
		testProtocol(AHBP_EX.class);

		testProtocol(DFCN.class);
//		testProtocol(Proba.class);
//		testProtocol(FreeDFCN.class);
//		testProtocol(PurelyProactiveDFCN.class);
//		testProtocol(PurelyReactiveDFCN.class);
//		testProtocol(FreeProactiveDFCN.class);
		println("Completed.");
	}


	/**
	 * For the given protocol, returns the evolution of each measure with the variation of the density.
	 * @param protocol
	 * @return
	 */
//	private Map<Measure, Function> testProtocol(Class protocol)
	private void testProtocol(Class protocol)
	{
		String protocolName = protocol.getName().substring(protocol.getName().lastIndexOf(".") + 1 );
		println("\ttesting protocol " + protocolName);

		Relation measure_value_function = new DefaultRelation(); 
		
		for (int i = 0; i < steps.length; ++i)
		{
			int stationDensity = steps[i];
			Map measure_value = testProtocol(protocol, stationDensity, tests);
			
			if (measure_value == null)
			{
			}
			else
			{
				Iterator measureIterator = measure_value.keySet().iterator();
				
				while (measureIterator.hasNext())
				{
					Sensor measure = (Sensor) measureIterator.next();
					Function function = (Function) measure_value_function.getValue(measure);
					
					if (function == null)
					{
						function = new Function();
						function.setName(measure.getName());
						measure_value_function.add(measure, function);
					}
					
					function.add(new Double(stationDensity), measure_value.get(measure));
				}
			}
		}

		Iterator measureIterator = measure_value_function.getKeys().iterator();
		
		while (measureIterator.hasNext())
		{
			Sensor measure = (Sensor) measureIterator.next();
			Function function = (Function) measure_value_function.getValue(measure);
			File file = getDataFile(new String[] {protocolName, measure.getName()}, new File("."));
			println("\t\twriting file " + file.getAbsolutePath());
			saveFunctionToFile(function, file);
		}
	}
			
	
	//Map<Measure><Value>
	private Map testProtocol(Class protocol, int density, int simulationCount)
	{
		println("\t\twith density=" + density);
		Relation measure_values = new DefaultRelation(); 

		for (int i = 1; i <= simulationCount; ++i)
		{
			println("\t\t\ttest " + i);
			Collection relevantMeasures = testProtocolOnce(protocol, density);
			
			if (relevantMeasures == null)
			{
				return null;
			}
			else
			{
				Iterator measureIterator = relevantMeasures.iterator();
				
				while (measureIterator.hasNext())
				{
					Sensor measure = (Sensor) measureIterator.next();
					measure_values.add(measure, measure.getLastValue());
				}
			}
		}

		println("\t\t\tProcessing average");
		Map measure_value = new HashMap();
		Iterator measureIterator = measure_values.getKeys().iterator();
		
		while (measureIterator.hasNext())
		{
			Sensor measure = (Sensor) measureIterator.next();
			Collection values = measure_values.getValues(measure);
			double average = Utilities.getAverage(values);
			double standardDeviation = Utilities.getStandardDeviation(values);
			measure_value.put(measure, new AvgAndStandDev(average, standardDeviation));
		}
		
		return measure_value;
	}

	private Collection testProtocolOnce(Class protocol, int density)
	{
		MadhocSimulation simulation = carryOutSimulation(protocol, density);
		
		if (simulation == null)
		{
			return null;
		}
		else
		{
			Collection relevantMeasures = new Vector();
			Iterator measureIterator = simulation.findAllSensors().iterator();
			
			while (measureIterator.hasNext())
			{
				Sensor measure = (Sensor) measureIterator.next();
				
				if (measure instanceof NumericalSensor)
				{
					if (measure.getLastValue() != null)
					{
						relevantMeasures.add(measure);
					}
				}
			}
			
			return relevantMeasures;
		}
	}
	
	private MadhocSimulation carryOutSimulation(Class protocol, int mobileStationDensity)
	{
		// creates the network, which is initially empty
		Network net = new Network();
		
		// define a natural environment for the network
		// this environment will have a great impact on the structure of the network
		// for instance, the city environment will incorporate walls that will prevent the
		// radio waves from propagating
		// here we define a 1 square kilometer (1 million square meter)
		// open area (which could be a field) 
		net.setNetworkEnvironment(new OpenAreaEnvironment(250000));
//		net.setEnvironment(new OpenAreaWithHotspotsEnvironment(250000, 100));

		// creates the stations that will populate the network
		net.createStations(mobileStationDensity);
		
		if (!Graph.findUnreachableNetworkTypes(net).isEmpty())
			System.out.println("UNREACHABLE");
		
		// the stations have been located randomly on the environment
		// it might be interested to let them run for a while in order
		// to stabilize the network graph
		net.getNetworkEnvironment().initializeNodesLocation(net.getAllComputers());

		// once we consider the location of the station has been initialized
		// we establish the connections that define the network itself
		net.createNewConnections();

		// the network will be handled by a simulation
		MadhocSimulation simulation = new MadhocSimulation();
		simulation.setName("test protocol" + protocol.getName());
		simulation.setNetwork(net);

		// the broadcasting application is what we want to run and observe


		// it is now time to define the applications that will run on the network
		{
			Collection applications = new Vector();

			{
				// the "core" application is a basic application that aim at extracting some
				// general purpose information on the simulation process
				Monitor app = new NetworkMonitor();
				app.getSensorMap().remove(NumberOfPartitions.class);
				app.getSensorMap().remove(NumberOfPartitions.class);
				app.getSensorMap().remove(DegreeDistributionMeasure.class);
				app.getSensorMap().remove(PartitionDistributionMeasure.class);
				app.getSensorMap().remove(LoadDistributionMeasure.class);
				app.getSensorMap().remove(ConnectionBandwithDistributionMeasure.class);

				applications.add(app);
			}
			{
				// define the broadcasting strategy we want to carry out
				BroadcastingMonitor broadcastingApp = new BroadcastingMonitor();
				broadcastingApp.setStationApplicationClass(protocol);
				broadcastingApp.setMode(BroadcastingMonitor.ONE_TO_ALL);
				broadcastingApp.setMessageLifetime(Long.MAX_VALUE);
				applications.add(broadcastingApp);
			}

			// the simulation knows the application is has to carry out
			// now let deploy them over the network
			simulation.deployApplications(applications);
		}

		// at this step the simulation is ready to be used
		// the user can get information on the simulation process via the Simulation.getMeasures() method,
		// via the methods provided by the various objects (Simulation, Network, Station...)
		// or via the discrete event file, depending on his own needs
		BroadcastingMonitor broadcastingApp = (BroadcastingMonitor) simulation.getMonitorMap().get(BroadcastingMonitor.class);

		
		while (broadcastingApp.getAverageCoverageRate() < 1)
		{
			if (simulation.getIteration() > 120 && ((NumericalSensor) broadcastingApp.getSensorMap().get(AverageNumberOfEmissionMeasure.class)).getVariationInTheLastIterations(120) == 0)
			{
				break;
			}

			simulation.iterate();
		}

		return simulation;
	}
}
