package org.lucci.madhoc.broadcast.gui;

import java.awt.Color;
import java.util.Collection;

import org.lucci.madhoc.broadcast.BroadcastingMonitor;
import org.lucci.madhoc.broadcast.BroadcastingProtocol;
import org.lucci.madhoc.broadcast.LocalInfo;
import org.lucci.madhoc.messaging.TransferableObject;
import org.lucci.madhoc.network.Station;
import org.lucci.madhoc.network.net.Location;
import org.lucci.up.data.DataElement;
import org.lucci.up.data.rendering.point.CirclePointRenderer;
import org.lucci.up.data.rendering.point.PointRenderer;
import org.lucci.up.data.rendering.point.TextPointRenderer;
import org.lucci.up.system.Space;


public class SingleBroadcastingMessageStationRenderer extends PointRenderer
{
	TextPointRenderer adsTextRenderer = new TextPointRenderer(); 
	CirclePointRenderer adsOvalRenderer = new CirclePointRenderer(); 

	public SingleBroadcastingMessageStationRenderer()
	{
		adsOvalRenderer.setRadius(3);
	}

	protected void drawImpl(DataElement dataElement, Space space)
	{
		Location point = (Location) dataElement;
		Station computer = point.getComputer();
		BroadcastingProtocol app = (BroadcastingProtocol) computer.getApplicationMap().getValue(BroadcastingProtocol.class);
		BroadcastingMonitor simApp = (BroadcastingMonitor) app.getMonitor();
		Collection hostedObjects = app.getHostedObjectsMap().keySet();

		// if we're doing a ONE to ALL
		if (simApp.getObjects().size() == 1)
		{
			if (hostedObjects.size() == 1)
			{
				TransferableObject object = (TransferableObject) hostedObjects.iterator().next();
				LocalInfo info = app.getLocalInfo(object);
				
				if (info.decision == -1)
				{
					adsOvalRenderer.setColor(Color.orange);
					adsOvalRenderer.setFillColor(Color.orange);
				}
				else if (info.decision == 0)
				{
					adsOvalRenderer.setColor(Color.red);
					adsOvalRenderer.setFillColor(Color.red);
				}
				else if (info.decision == 1)
				{
					// if the emission has not occured yet
					if (info.emissionCount == 0)
					{
						adsOvalRenderer.setColor(Color.green);
						adsOvalRenderer.setFillColor(null);
					}
					else
					{
						adsOvalRenderer.setColor(Color.green);
						adsOvalRenderer.setFillColor(Color.green);
					}
				}
				else
				{
					throw new IllegalStateException("invalid state " + info.decision);
				}
			}
			else
			{
				adsOvalRenderer.setColor(Color.black);
				adsOvalRenderer.setFillColor(Color.white);
			}

			adsOvalRenderer.drawImpl(computer.getLocation(), space);
		}
		else
		{
			adsTextRenderer.setText(String.valueOf(hostedObjects.size()));
			adsTextRenderer.drawImpl(computer.getLocation(), space);
		}
	}
	public String getPublicName()
	{
		return "single broadcast message";
	}
}
