package org.lucci.madhoc.broadcast.gui;

import java.awt.Color;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.lucci.madhoc.broadcast.BroadcastingMonitor;
import org.lucci.madhoc.broadcast.BroadcastingProtocol;
import org.lucci.madhoc.broadcast.impl.research.dfcn.CABP;
import org.lucci.madhoc.messaging.TransferableObject;
import org.lucci.madhoc.network.Station;
import org.lucci.madhoc.network.net.Location;
import org.lucci.up.data.DataElement;
import org.lucci.up.data.rendering.point.PointRenderer;
import org.lucci.up.system.Dimension;
import org.lucci.up.system.Space;


public class UrgencyStationRenderer extends PointRenderer
{
	private List allObjectsList;
	
	protected void drawImpl(DataElement dataElement, Space space)
	{
		Location point = (Location) dataElement;
		Station station = point.getComputer();
		final BroadcastingProtocol protocol = (BroadcastingProtocol) station.getApplicationMap().getValue(BroadcastingProtocol.class);

		if (allObjectsList == null)
		{
			BroadcastingMonitor app = (BroadcastingMonitor) protocol.getMonitor();
			allObjectsList = new Vector(app.getObjects());
			Collections.sort(allObjectsList, new Comparator()
				{
					public int compare(Object object0, Object object1)
					{
						CABP.CABPBroadcastInformation bi0 = (CABP.CABPBroadcastInformation) protocol.getLocalInfo((TransferableObject) object0).message.getMessageInformation();
						CABP.CABPBroadcastInformation bi1 = (CABP.CABPBroadcastInformation) protocol.getLocalInfo((TransferableObject) object1).message.getMessageInformation();
						return new Double(bi0.getUrgency()).compareTo(new Double(bi1.getUrgency()));
					}
				});
			System.out.println(allObjectsList.size());
		}

		Dimension xDimension = space.getXDimension();
		Dimension yDimension = space.getYDimension();

		int edge = allObjectsList.size() * 4;
		int msgLineHeight = (int) (((double) edge) / ((double) allObjectsList.size()));
		int x = xDimension.convertToGraphicsCoordonateSystem(point.getX());
		int y = yDimension.convertToGraphicsCoordonateSystem(point.getY());

		Iterator objects = protocol.getHostedObjectsMap().keySet().iterator();
		
		while (objects.hasNext())
		{
			TransferableObject object = (TransferableObject) objects.next();
			int index = allObjectsList.indexOf(object);

			if (index == -1)
				throw new IllegalStateException();
	
			CABP.CABPBroadcastInformation bi0 = (CABP.CABPBroadcastInformation) protocol.getLocalInfo(object).broadcastInformation;
			space.getFigureGraphics().setColor(getColor(object, bi0));
			space.getFigureGraphics().fillRect(x - edge / 2, y - edge / 2 + index * msgLineHeight, edge, msgLineHeight);
		}

		space.getFigureGraphics().setColor(Color.black);
		space.getFigureGraphics().fillRect(x - 1, y - 1, 3, 3);
	}

	private Color getColor(TransferableObject o, CABP.CABPBroadcastInformation i)
	{
		return new Color((int) (255 * i.getUrgency()), 0, (int) (255 * -i.getUrgency() + 255));
	}
	
    public String getPublicName()
    {
        return "urgency";
    }
}
