package org.lucci.madhoc.broadcast.impl.research;

import java.util.Iterator;

import org.lucci.madhoc.broadcast.BroadcastInformation;
import org.lucci.madhoc.broadcast.LocalInfo;
import org.lucci.madhoc.broadcast.RandomDelayBasedBroadcastingProtocol;
import org.lucci.madhoc.env.mall.MallMobility;


/**
 * @author luc.hogie
 * Created on May 24, 2004
 * 
 * Cannot emit a message more than getMaximumEmissionCount() times.
 * If an incoming message has already previously received, it is not emitted and goes directly to sleep...
 * One a message is emitted, it sleeps... zzzZZZ zzzZZZ... And wakes up each getSleepingDuration() minutes.
 * When the hosting station reach a high density area, all ads it hosts are awaken and reemitted.
 */
public class LightWeight
	extends RandomDelayBasedBroadcastingProtocol
{
	// tells if the station was within a concentration area
	// 0 = no, 1 = yes, -1 = dunno
	private int stationWasWithinAConcentrationArea = -1;
	
	private long sleepingDuration = 1000 * 60;
	private int maximumEmissionCount = 1;



	/**
	 * @param ad
	 */
	protected void takeDecision(BroadcastInformation ad)
	{
		LocalInfo info = getLocalInfo(ad); 

		// if the station has sent the ad more than the maximum emission number, it doesn't send it anymore
		if (info.emissionCount > getMaximumEmissionCount())
		{
			info.decision = 0;
		}
		else
		{
			info.decision = 1;
		}
	}



	
	public void doIt(double allocatedTime)
	{
		super.doIt(allocatedTime);

		MallMobility mobility = (MallMobility) getComputer().getMobilityModel();

		// if the logical location of the mobile station has not yet been defined, do it now!
		if (stationWasWithinAConcentrationArea == -1)
		{
			stationWasWithinAConcentrationArea = mobility.isWithinItsShop() ? 1 : 0; 
		}
		else
		{
			// if the mobile station has reached or left a concentration area
			if (mobility.isWithinItsShop() != (stationWasWithinAConcentrationArea == 1))
			{
				stationWasWithinAConcentrationArea = mobility.isWithinItsShop() ? 1 : 0;
				awakeAllMessages();
			}
		}
	}

	protected void awakeAllMessages()
	{
		Iterator hostedAdIterator = getHostedObjectsMap().keySet().iterator();

		while (hostedAdIterator.hasNext())
		{
			BroadcastInformation ad = (BroadcastInformation) hostedAdIterator.next();
			LocalInfo info = getLocalInfo(ad);
			
			// decides to broadcast the ad immediately
			// since there is not risk of collision, the rad is of no use
			info.retransmissionDate = getSimulatedTime();
		}
	}

	protected void spore(BroadcastInformation msg)
	{
		LocalInfo info = getLocalInfo(msg);
		info.retransmissionDate = getSimulatedTime() + getSleepingDuration();
	}
	

	protected void adJustReceived(BroadcastInformation msg)
	{
		super.messageJustReceived(msg);
		LocalInfo info = getLocalInfo(msg);
		
		// the ad has already been received previously
		if (info.redundantReceptionCount > 0)
		{
			spore(msg);
		}
	}


	protected void adJustBeingSent(BroadcastInformation msg)
	{
		// each message sent is ask to sleep
		spore(msg);
	}


	/* (non-Javadoc)
	 * @see org.lucci.madhoc.network.StationApplication#getName()
	 */
	public String getName()
	{
		return "Lightweight";
	}


	/* (non-Javadoc)
	 * @see org.lucci.madhoc.broadcast.BroadcastingProtocol#isMultipleRebroadcastAllowed()
	 */
	public boolean isMultipleRebroadcastAllowed()
	{
		return true;
	}

	public int getMaximumEmissionCount()
	{
		return maximumEmissionCount;
	}
	
	public void setMaximumEmissionCount(int maximumEmissionCount)
	{
		if (maximumEmissionCount < 0)
			throw new IllegalArgumentException();

		this.maximumEmissionCount = maximumEmissionCount;
	}
	public long getSleepingDuration()
	{
		return sleepingDuration;
	}
	public void setSleepingDuration(long sleepingDuration)
	{
		if (sleepingDuration < 0)
			throw new IllegalArgumentException();

		this.sleepingDuration = sleepingDuration;
	}
}
