package org.lucci.madhoc.broadcast.impl.research;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import org.lucci.madhoc.broadcast.BroadcastInformation;
import org.lucci.madhoc.broadcast.LocalInfo;
import org.lucci.madhoc.broadcast.impl.research.dfcn.DFCN.DFCNInfo;
import org.lucci.madhoc.broadcast.impl.standard.ProbabilisticScheme;
import org.lucci.madhoc.broadcast.impl.standard.FloodingWithSelfPruning.FWSPLocalInfo;

/*
 * Created on Jan 22, 2005
 */

/**
 * @author luc.hogie
 */
public class Proba extends ProbabilisticScheme
{
	protected static class ProbaInfo extends LocalInfo
	{
		public Collection knownHosts = new HashSet();
		
		public int getSizeInBytes()
		{
			return super.getSizeInBytes() + knownHosts.size() * 4;
		}
	}

	
	public double getBroadcastingProbability(BroadcastInformation msg)
	{
		Collection unreachedNeighbors = getUnreachedHosts(msg);
		double gain = (double) unreachedNeighbors.size() / (double) getComputer().getNeighborhood().size();
		return Math.sqrt(gain);
	}

	public boolean isMultipleRebroadcastAllowed()
	{
		return true;
	}

	public String getName()
	{
		return "Proba";
	}
	
	
	
	protected LocalInfo instantiateLocalInfo()
	{
		return new ProbaInfo();
	}
	protected void adJustReceived(BroadcastInformation ad)
	{
		super.messageJustReceived(ad);
		ProbaInfo info = (ProbaInfo) getLocalInfo(ad);
		info.knownHosts.addAll(ad.getMessage().getSourceStationApplication().getComputer().getNeighborhood());
		info.knownHosts.add(ad.getMessage().getSourceStationApplication().getComputer());
	}
	
	public Collection getUnreachedHosts(BroadcastInformation msg)
	{
		ProbaInfo info = (ProbaInfo) getLocalInfo(msg);
		Collection c = new HashSet(getComputer().getNeighborhood());
		c.removeAll(info.knownHosts);
		return c;
	}
	
	protected Collection getKnownHosts(BroadcastInformation msg)
	{
		ProbaInfo info = (ProbaInfo) getLocalInfo(msg);
		return info.knownHosts;
	}

}
