package org.lucci.madhoc.broadcast.impl.research;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import org.lucci.madhoc.broadcast.BroadcastInformation;
import org.lucci.madhoc.broadcast.LocalInfo;
import org.lucci.madhoc.broadcast.RandomDelayBasedBroadcastingProtocol;
import org.lucci.madhoc.broadcast.impl.research.Proba.ProbaInfo;
import org.lucci.madhoc.network.MobileNode;

/*
 * Created on Jan 24, 2005
 */

/**
 * @author luc.hogie
 */
public class Topo extends RandomDelayBasedBroadcastingProtocol
{
	protected static class TopoInfo extends LocalInfo
	{
		public Collection knownHosts = new HashSet();
		
		public int getSizeInBytes()
		{
			return super.getSizeInBytes() + knownHosts.size() * 4;
		}
	}

	public String getName()
	{
		return "Topo";
	}

	/* (non-Javadoc)
	 * @see org.lucci.madhoc.broadcast.RandomDelayBasedBroadcastingProtocol#takeDecision(org.lucci.madhoc.broadcast.BroadcastMessage)
	 */
	protected void takeDecision(BroadcastInformation ad)
	{
		TopoInfo info = (TopoInfo) getLocalInfo(ad);
		Iterator ni = getComputer().getNeighborhood().iterator();

		double sum = 0;
		double max = 0;
		
		while (ni.hasNext())
		{
			MobileNode n = (MobileNode) ni.next();
			Collection exclusiveNeighbors = getExclusiveNeighbors(n);
			sum += exclusiveNeighbors.size();
			
			if (exclusiveNeighbors.size() > max)
			{
				max = exclusiveNeighbors.size();
			}
		}
		
		double avg = sum / getComputer().getNeighborhood().size();
		System.out.println(avg);
		info.decision = avg > 0.5  ? 1 : 0;
	}
	
	Collection getExclusiveNeighbors(MobileNode n)
	{
		Collection en = new HashSet(n.getNeighborhood());
		en.removeAll(getComputer().getNeighborhood());
		
		Iterator i = getComputer().getNeighborhood().iterator();
		
		while (i.hasNext())
		{
			MobileNode nn = (MobileNode) i.next();
			
			if (nn != n)
			{
				en.removeAll(nn.getNeighborhood());
			}
		}
		
		return en;
	}

	/* (non-Javadoc)
	 * @see org.lucci.madhoc.broadcast.BroadcastingProtocol#adJustBeingSent(org.lucci.madhoc.broadcast.BroadcastMessage)
	 */
	protected void adJustBeingSent(BroadcastInformation ad)
	{
	}

	/* (non-Javadoc)
	 * @see org.lucci.madhoc.broadcast.BroadcastingProtocol#isMultipleRebroadcastAllowed()
	 */
	protected boolean isMultipleRebroadcastAllowed()
	{
		return false;
	}

	protected LocalInfo instantiateLocalInfo()
	{
		return new TopoInfo();
	}
	
	protected void adJustReceived(BroadcastInformation ad)
	{
		super.messageJustReceived(ad);
		TopoInfo info = (TopoInfo) getLocalInfo(ad);
		info.knownHosts.addAll(ad.getMessage().getSourceStationApplication().getComputer().getNeighborhood());
		info.knownHosts.add(ad.getMessage().getSourceStationApplication().getComputer());
	}
	
	public Collection getUnreachedHosts(BroadcastInformation msg)
	{
		ProbaInfo info = (ProbaInfo) getLocalInfo(msg);
		Collection c = new HashSet(getComputer().getNeighborhood());
		c.removeAll(info.knownHosts);
		return c;
	}
	
	protected Collection getKnownHosts(BroadcastInformation msg)
	{
		ProbaInfo info = (ProbaInfo) getLocalInfo(msg);
		return info.knownHosts;
	}
}
