/*
 * Created on Apr 8, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package org.lucci.madhoc.broadcast.impl.research.dfcn;

import java.awt.Color;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import org.lucci.madhoc.broadcast.BroadcastInformation;
import org.lucci.madhoc.broadcast.LocalInfo;
import org.lucci.madhoc.broadcast.impl.standard.FloodingWithSelfPruning;
import org.lucci.madhoc.messaging.Message;
import org.lucci.madhoc.messaging.TransferableObject;

/**
 * @author luc.hogie
 */
public class CABP
	extends FloodingWithSelfPruning
{
	private boolean rebroadcastingOnNewConnectionEnabled = true;
	
	protected static class CABPLocalInfo extends LocalInfo
	{
		public Collection knownHosts = new HashSet();
		
		public int getSizeInBytes()
		{
			return super.getSizeInBytes() + knownHosts.size() * 4;
		}
	}


	// the ad needs to carry its urgency
	public static class CABPBroadcastInformation extends FloodingWithSelfPruning.FWSPBroadcastInfo
	{
		private double urgency = 0.5;
		private Color color;

		public void setUrgency(double urgency)
		{
			if (urgency < 0)
				throw new IllegalArgumentException("urgency < 0");

			if (urgency > 1)
				throw new IllegalArgumentException("urgency > 1");

			this.urgency = urgency;
			color = new Color((int) (urgency * 255), 0, (int) ((1 - urgency) * 255));
		}

		public int getSizeInBytes()
		{
			return super.getSizeInBytes() + 1;
		}

		public Object clone()
		{
			CABPBroadcastInformation clone = (CABPBroadcastInformation) super.clone();
			clone.setUrgency(getUrgency());
			return clone;
		}
		public Color getColor()
		{
			return color;
		}
		public double getUrgency()
		{
			return urgency;
		}
	}

	protected BroadcastInformation instantiateBroadcastInformation()
	{
		CABPBroadcastInformation m = new CABPBroadcastInformation();
		m.setUrgency(getMonitor().getNetwork().getSimulation().getRandomNumberGenerator().getRandomDouble());
		return m;
	}



	public void doIt(double time)
	{
		// if there are some new connections, all ads need to be rebroadcasted
		if (isRebroadcastingOnNewConnectionEnabled() && !getHostedObjectsMap().isEmpty() && !getComputer().getNetworkingUnit().getNewConnections().isEmpty())
		{
			Iterator i = getHostedObjectsMap().keySet().iterator();

			while (i.hasNext())
			{
				TransferableObject object = (TransferableObject) i.next();
				CABPLocalInfo localInfo = (CABPLocalInfo) getLocalInfo(object);
				CABPBroadcastInformation broadcastInfo = (CABPBroadcastInformation) localInfo.broadcastInformation;

				if (getComputer().getNetworkingUnit().getNeighborhood().size() <= getMininumDegreeForDenseNetwork(broadcastInfo))
				{
					takeDecision(object);

					if (localInfo.decision == 1)
					{
						localInfo.sendStrategy = LocalInfo.PROACTIVE;
						localInfo.retransmissionDate = getSimulatedTime();
					}
				}
			}
		}
	
		super.doIt(time);
	}	

	
	
	protected LocalInfo instantiateLocalInfo()
	{
		return new CABPLocalInfo();
	}

	protected void messageJustReceived(Message msg)
	{
		CABPLocalInfo info = (CABPLocalInfo) getLocalInfo(msg.getContent());
		CABPBroadcastInformation broadcastInfo = (CABPBroadcastInformation) info.broadcastInformation;
		setMaximumDelay((long) ((info.redundantReceptionCount + 1) * (10 - 9 * broadcastInfo.getUrgency())));
		super.messageJustReceived(msg);
		info.knownHosts.addAll(broadcastInfo.senderNeighbors);
		info.knownHosts.add(msg.getSourceStationApplication().getComputer());
	}

	protected void messageJustBeingSent(Message msg)
	{
		super.messageJustBeingSent(msg);
		LocalInfo info = getLocalInfo(msg.getContent());
		info.retransmissionDate = Long.MAX_VALUE;
	}

	protected void takeDecision(TransferableObject ad)
	{
		LocalInfo info = getLocalInfo(ad);
		
		if (info.emissionCount < getMaximumNumberOfEmissions(info))
		{
			super.takeDecision(ad);
		}
		else
		{
			info.decision = 0;
		}
	}


	protected int getDecision(BroadcastInformation msg, Collection newHosts)
	{
		if (getComputer().getNetworkingUnit().getNeighborhood().size() < 4)
		{
			return super.getDecision(msg, newHosts);
		}
		else
		{
			double benefit = (double) newHosts.size() / (double) getComputer().getNetworkingUnit().getNeighborhood().size();
			return benefit > getAcceptableBenefit((CABPBroadcastInformation) msg) ? 1 : 0;
		}
	}

	private double getMaximumNumberOfEmissions(LocalInfo info)
	{
//		return (5 - info.redundantReceptionCount * (1 - getUrgency())) / (1 - getUrgency());
		return 5;
	}

	protected Collection getKnownHosts(TransferableObject object)
	{
		CABPLocalInfo info = (CABPLocalInfo) getLocalInfo(object);
		return info.knownHosts;
	}


	/* (non-Javadoc)
	 * @see org.lucci.madhoc.network.StationApplication#getName()
	 */
	public String getName()
	{
		return "CABP";
	}

	protected double getAcceptableBenefit(CABPBroadcastInformation msg)
	{
//		double t = -getUrgency() / 2 + 0.5;
		double t = 0.8 * (1 - msg.getUrgency());
//
//		if (t < 0 || t > 0.5)
//			throw new IllegalStateException("benefit = " + t);
		
		return t;
	}

	private double getMininumDegreeForDenseNetwork(CABPBroadcastInformation broadcastInfo)
	{
		double d = Math.pow(-broadcastInfo.getUrgency() + 1.0, -2.0) + 3.0;
		return d;
	}

	

	public boolean isMultipleRebroadcastAllowed()
	{
		return isRebroadcastingOnNewConnectionEnabled();
	}

	public boolean isRebroadcastingOnNewConnectionEnabled()
	{
		return rebroadcastingOnNewConnectionEnabled;
	}

	public void setRebroadcastingOnNewConnectionEnabled(boolean rebroadcastingOnNewConnectionEnabled)
	{
		this.rebroadcastingOnNewConnectionEnabled = rebroadcastingOnNewConnectionEnabled;
	}
}
