/*
 * Created on Apr 8, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package org.lucci.madhoc.broadcast.impl.research.dfcn;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import org.lucci.madhoc.broadcast.BroadcastInformation;
import org.lucci.madhoc.broadcast.LocalInfo;
import org.lucci.madhoc.broadcast.impl.standard.FloodingWithSelfPruning;
import org.lucci.madhoc.messaging.Message;
import org.lucci.madhoc.messaging.TransferableObject;
import org.lucci.madhoc.network.Station;
import org.lucci.madhoc.simulation.Simulation;

/**
 * @author luc.hogie
 */
public class DFCN
	extends FloodingWithSelfPruning
{
	private double minimumGain = 0.5;
	private boolean rebroadcastingOnNewConnectionEnabled = true;
	private int degreeUntilWhichFWSPIsApplied = 4;
	private int degreeUntilWhichRebroadcastOnNewConnectionOccurs = 5;

	protected static class DFCNInfo extends LocalInfo
	{
		public Collection<Station> knownHosts = new HashSet<Station>();
		
		public int getSizeInBytes()
		{
			return super.getSizeInBytes() + knownHosts.size() * 4;
		}
	}



	protected LocalInfo instantiateLocalInfo()
	{
		return new DFCNInfo();
	}

	public void configure()
		throws Throwable
	{
		super.configure();
		Simulation simulation = getMonitor().getNetwork().getSimulation();
		setMinimumAcceptableBenefit(simulation.getConfiguration().getDouble("dfcn_acceptable_benefit"));
		setDegreeUntilWhichFWSPIsApplied(simulation.getConfiguration().getInteger("degree_until_which_fwsp_is_applied"));
		setDegreeUntilWhichRebroadcastOnNewConnectionOccurs(simulation.getConfiguration().getInteger("degree_until_which_rebroadcast_on_new_connection_occurs"));
		setRebroadcastingOnNewConnectionEnabled(simulation.getConfiguration().getBoolean("dfcn_enable_proactive_behavior"));
	}
	
	public void doIt(double time)
	{
		// if there are some new connections, all ads need to be rebroadcasted
		if (isRebroadcastingOnNewConnectionEnabled())
		{
			if (getComputer().getNetworkingUnit().getNeighborhood().size() < getDegreeUntilWhichRebroadcastOnNewConnectionOccurs())
			{
				if (!getHostedObjectsMap().isEmpty())
				{
					if (!getComputer().getNetworkingUnit().getNewConnections().isEmpty())
					{
						Iterator i = getHostedObjectsMap().keySet().iterator();
	
						while (i.hasNext())
						{
							TransferableObject object = (TransferableObject) i.next();
							DFCNInfo info = (DFCNInfo) getLocalInfo(object);
							takeDecision(object);
							
							if (info.decision == 1)
							{
								info.sendStrategy = LocalInfo.PROACTIVE;
								info.retransmissionDate = getSimulatedTime();
							}
						}
					}
				}
			}
		}
	
		super.doIt(time);
	}
	
	protected void messageJustReceived(Message msg)
	{
		super.messageJustReceived(msg);
		DFCNInfo info = (DFCNInfo) getLocalInfo(msg.getContent());
		FWSPBroadcastInfo broadcastInfo = (FWSPBroadcastInfo) msg.getMessageInformation();
		info.knownHosts.addAll(broadcastInfo.senderNeighbors);
		info.knownHosts.add(msg.getSourceStationApplication().getComputer());
	}

	protected void messageJustBeingSent(Message msg)
	{
		super.messageJustBeingSent(msg);
		LocalInfo info = getLocalInfo(msg.getContent());
		info.retransmissionDate = Long.MAX_VALUE;
	}

	protected int getDecision(BroadcastInformation msg, Collection newHosts)
	{
		if (getComputer().getNetworkingUnit().getNeighborhood().size() < getDegreeUntilWhichFWSPIsApplied())
		{
			return super.getDecision(msg, newHosts);
		}
		else
		{
			double gain = (double) newHosts.size() / (double) getComputer().getNetworkingUnit().getNeighborhood().size();
			return gain >= getMinimumGain() ? 1 : 0;
		}
	}


	protected Collection getKnownHosts(TransferableObject object)
	{
		DFCNInfo info = (DFCNInfo) getLocalInfo(object);
		return info.knownHosts;
	}


	
	/* (non-Javadoc)
	 * @see org.lucci.madhoc.network.StationApplication#getName()
	 */
	public String getName()
	{
		return "DFCN";
	}

	public double getMinimumGain()
	{
		return minimumGain;
	}

	public void setMinimumAcceptableBenefit(double minimumGain)
	{
		if (minimumGain < 0)
			throw new IllegalArgumentException("minimum gain is < 0");

		if (minimumGain > 1)
			throw new IllegalArgumentException("minimum gain is > 1");

		this.minimumGain = minimumGain;
	}

	public int getDegreeUntilWhichFWSPIsApplied()
	{
		return degreeUntilWhichFWSPIsApplied;
	}
	
	public void setDegreeUntilWhichFWSPIsApplied(int degreeUntilWhichFWSPIsApplied)
	{
		if (degreeUntilWhichFWSPIsApplied < 0)
			throw new IllegalArgumentException("degreeUntilWhichFWSPIsApplied < 0");
	
		this.degreeUntilWhichFWSPIsApplied = degreeUntilWhichFWSPIsApplied;
	}

	public boolean isMultipleRebroadcastAllowed()
	{
		return isRebroadcastingOnNewConnectionEnabled();
	}

	public boolean isRebroadcastingOnNewConnectionEnabled()
	{
		return rebroadcastingOnNewConnectionEnabled;
	}

	public void setRebroadcastingOnNewConnectionEnabled(boolean rebroadcastingOnNewConnectionEnabled)
	{
		this.rebroadcastingOnNewConnectionEnabled = rebroadcastingOnNewConnectionEnabled;
	}
	public int getDegreeUntilWhichRebroadcastOnNewConnectionOccurs()
	{
		return degreeUntilWhichRebroadcastOnNewConnectionOccurs;
	}
	public void setDegreeUntilWhichRebroadcastOnNewConnectionOccurs(int degreeUntilWhichRebroadcastOnNewConnectionOccurs)
	{
		if (degreeUntilWhichRebroadcastOnNewConnectionOccurs < 0)
			throw new IllegalArgumentException();

		this.degreeUntilWhichRebroadcastOnNewConnectionOccurs = degreeUntilWhichRebroadcastOnNewConnectionOccurs;
	}
}
