/*
 * Created on Apr 8, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package org.lucci.madhoc.broadcast.impl.research.dfcn.test;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import org.lucci.madhoc.broadcast.BroadcastInformation;
import org.lucci.madhoc.broadcast.LocalInfo;
import org.lucci.madhoc.broadcast.impl.standard.FloodingWithSelfPruning;

/**
 * @author luc.hogie
 */
public class SDFCN
	extends FloodingWithSelfPruning
{
	public static class SDFCN_00 extends SDFCN
	{
		public SDFCN_00()
		{
			super(0);
		}
	}
	public static class SDFCN_02 extends SDFCN
	{
		public SDFCN_02()
		{
			super(0.2);
		}
	}
	public static class SDFCN_04 extends SDFCN
	{
		public SDFCN_04()
		{
			super(0.4);
		}
	}
	public static class SDFCN_06 extends SDFCN
	{
		public SDFCN_06()
		{
			super(0.6);
		}
	}
	public static class SDFCN_08 extends SDFCN
	{
		public SDFCN_08()
		{
			super(0.8);
		}
	}
	public static class SDFCN_10 extends SDFCN
	{
		public SDFCN_10()
		{
			super(1);
		}
	}
	
	private double urgency;
	private boolean rebroadcastingOnNewConnectionEnabled = true;
	
	protected static class DFCNInfo extends LocalInfo
	{
		public Collection knownHosts = new HashSet();
		
		public int getSizeInBytes()
		{
			return super.getSizeInBytes() + knownHosts.size() * 4;
		}
	}


	public SDFCN()
	{
		this(0);
	}

	public SDFCN(double urgency)
	{
		setUrgency(urgency);
//		System.out.println("acceptable benefit: " + getAcceptableBenefit());
//		System.out.println("max degree: " + getMininumDegreeForDenseNetwork());
	}


	protected LocalInfo instantiateLocalInfo()
	{
		return new DFCNInfo();
	}

	public void configure()
		throws Throwable
	{
		super.configure();
		setUrgency((Double.valueOf((String) simulation.getConfiguration().getConfigurationValue("dfcn_urgency"))).doubleValue());
		setRebroadcastingOnNewConnectionEnabled(Boolean.valueOf((String) simulation.getConfiguration().getConfigurationValue("dfcn_enable_proactive_behavior")).booleanValue());
	}
	
	public void doIt(double time)
	{
		// if there are some new connections, all ads need to be rebroadcasted
		if (isRebroadcastingOnNewConnectionEnabled())
		{
			if (getComputer().getNeighborhood().size() <= getMininumDegreeForDenseNetwork())
			{
				if (!getHostedObjectsMap().isEmpty())
				{
					if (!getComputer().getNewConnections().isEmpty())
					{
						Iterator i = getHostedObjectsMap().keySet().iterator();
	
						while (i.hasNext())
						{
							BroadcastInformation m = (BroadcastInformation) i.next();
							DFCNInfo info = (DFCNInfo) getLocalInfo(m);
							takeDecision(m);
							
							if (info.decision == 1)
							{
								info.sendStrategy = LocalInfo.PROACTIVE;
								info.retransmissionDate = getSimulatedTime();
							}
						}
					}
				}
			}
		}
	
		super.doIt(time);
	}	
	protected void adJustReceived(BroadcastInformation msg)
	{
		super.messageJustReceived(msg);
		FWSPLocalInfo fwspMsg = (FWSPLocalInfo) msg;
		DFCNInfo info = (DFCNInfo) getLocalInfo(fwspMsg);
		info.knownHosts.addAll(fwspMsg.senderNeighbors);
		info.knownHosts.add(fwspMsg.getMessage().getSourceStationApplication().getComputer());
	}

	protected void adJustBeingSent(BroadcastInformation ad)
	{
		super.messageJustBeingSent(ad);
		LocalInfo info = getLocalInfo(ad);
		info.retransmissionDate = Long.MAX_VALUE;
	}

	protected int getDecision(Collection newHosts)
	{
		if (getComputer().getNeighborhood().size() < 4)
		{
			return super.getDecision(newHosts);
		}
		else
		{
			double benefit = (double) newHosts.size() / (double) getComputer().getNeighborhood().size();
			return benefit > getAcceptableBenefit() ? 1 : 0;
		}
	}


	protected Collection getKnownHosts(FWSPLocalInfo msg)
	{
		DFCNInfo info = (DFCNInfo) getLocalInfo(msg);
		return info.knownHosts;
	}

	
	/* (non-Javadoc)
	 * @see org.lucci.madhoc.network.StationApplication#getName()
	 */
	public String getName()
	{
		return "SDFCN(" + getAcceptableBenefit() + ")";
	}

	protected double getAcceptableBenefit()
	{
//		double t = -getUrgency() / 2 + 0.5;
		double t = 0.8*(1-getUrgency());
//
//		if (t < 0 || t > 0.5)
//			throw new IllegalStateException("benefit = " + t);
		
		return t;
	}

	private double getMininumDegreeForDenseNetwork()
	{
		double d = Math.pow(-getUrgency() + 1.0, -2.0) + 3.0;
		return d;
	}

	
	public double getUrgency()
	{
		return urgency;
	}
	
	public void setUrgency(double urgency)
	{
		if (urgency < 0)
			throw new IllegalArgumentException("urgency < 0");

		if (urgency > 1)
			throw new IllegalArgumentException("urgency > 1");

		this.urgency = urgency;
	}

	public boolean isMultipleRebroadcastAllowed()
	{
		return isRebroadcastingOnNewConnectionEnabled();
	}

	public boolean isRebroadcastingOnNewConnectionEnabled()
	{
		return rebroadcastingOnNewConnectionEnabled;
	}

	public void setRebroadcastingOnNewConnectionEnabled(boolean rebroadcastingOnNewConnectionEnabled)
	{
		this.rebroadcastingOnNewConnectionEnabled = rebroadcastingOnNewConnectionEnabled;
	}
}
