/*
 * Created on Mar 29, 2004
 */
package org.lucci.madhoc.broadcast.impl.standard;

import java.util.Collection;
import java.util.HashSet;
import java.util.Vector;

import org.lucci.madhoc.broadcast.BroadcastInformation;
import org.lucci.madhoc.broadcast.LocalInfo;
import org.lucci.madhoc.broadcast.RandomDelayBasedBroadcastingProtocol;
import org.lucci.madhoc.messaging.Message;
import org.lucci.madhoc.messaging.TransferableObject;
import org.lucci.madhoc.network.Station;

public class FloodingWithSelfPruning
	extends RandomDelayBasedBroadcastingProtocol
{
	// the ad needs to carry the list of neighbors of its sender
	public static class FWSPBroadcastInfo extends BroadcastInformation
	{
		private Station sender;
		public Collection<Station> senderNeighbors = new Vector<Station>();

		public int getSizeInBytes()
		{
			return super.getSizeInBytes() + 4 + senderNeighbors.size() * 4;
		}

		public Object clone()
		{
			FWSPBroadcastInfo clone = (FWSPBroadcastInfo) super.clone();
			clone.senderNeighbors.addAll(senderNeighbors);
			return clone;
		}
	}

	protected BroadcastInformation instantiateBroadcastInformation()
	{
		return new FWSPBroadcastInfo();
	}

	
	/*
	 * Decides to rebroadcast only if at least one station can be reached
	 */
	protected int getDecision(TransferableObject msg, Collection newHosts)
	{
		return newHosts.isEmpty() ? 0 : 1;
	}

	protected void messageJustBeingSent(Message msg)
	{
		FWSPBroadcastInfo fwspAd = (FWSPBroadcastInfo) msg.getMessageInformation();
		fwspAd.sender = getComputer();
		fwspAd.senderNeighbors = new HashSet(getComputer().getNetworkingUnit().getNeighborhood());
	}

	/* (non-Javadoc)
	 * @see org.lucci.madhoc.network.StationApplication#getName()
	 */
	public String getName()
	{
		return "FWSP";
	}


	/* (non-Javadoc)
	 * @see org.lucci.madhoc.broadcast.BroadcastingProtocol#isMultipleRebroadcastAllowed()
	 */
	public boolean isMultipleRebroadcastAllowed()
	{
		return false;
	}


	/* (non-Javadoc)
	 * @see org.lucci.madhoc.broadcast.RandomDelayBasedBroadcastingProtocol#takeDecision(org.lucci.madhoc.broadcast.BroadcastMessage)
	 */
	protected void takeDecision(TransferableObject object)
	{
		LocalInfo localInfo = getLocalInfo(object);
		FWSPBroadcastInfo broadcastInfo = (FWSPBroadcastInfo) localInfo.broadcastInformation;
		localInfo.decision = getDecision(broadcastInfo, getPotentialNewHosts(object));
	}

	private Collection<Station> getPotentialNewHosts(TransferableObject object)
	{
		Collection<Station> newHosts = new HashSet<Station>(getComputer().getNetworkingUnit().getNeighborhood());
		newHosts.removeAll(getKnownHosts(object));
		return newHosts;
	}
	
	protected Collection<Station> getKnownHosts(TransferableObject object)
	{
		LocalInfo localInfo = getLocalInfo(object);
        
        if (localInfo.message != null)
        {
            FWSPBroadcastInfo broadcastInfo = (FWSPBroadcastInfo) localInfo.message.getMessageInformation();
            Collection<Station> c = new HashSet<Station>(broadcastInfo.senderNeighbors);
            c.add(broadcastInfo.sender);
            return c;
        }
        else
        {
            return new Vector<Station>();
        }
	}
}
