package org.lucci.madhoc.broadcast.impl.standard;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import org.lucci.madhoc.broadcast.LocalInfo;
import org.lucci.madhoc.broadcast.RandomDelayBasedBroadcastingProtocol;
import org.lucci.madhoc.messaging.Message;
import org.lucci.madhoc.messaging.TransferableObject;
import org.lucci.madhoc.network.Station;

/**
 * @author luc.hogie
 * Created on Jun 22, 2004
 */
public class LocationBasedScheme
	extends RandomDelayBasedBroadcastingProtocol
{
	protected static class DistanceAdInfo extends LocalInfo
	{
		public Collection sourceStations = new HashSet();

		public int getSizeInBytes()
		{
			return super.getSizeInBytes() + sourceStations.size() * 4;
		}
	}

	
	
	private double distanceThreshold = 5;
	
	protected LocalInfo instantiateLocalInfo()
	{
		return new DistanceAdInfo();
	}


	protected void messageJustReceived(Message ad)
	{
		super.messageJustReceived(ad);
		DistanceAdInfo localInfo = (DistanceAdInfo) getLocalInfo(ad.getContent());
		localInfo.sourceStations.add(ad.getSourceStationApplication().getComputer());
	}

	protected void messageJustBeingSent(Message ad)
	{
	}

	protected void takeDecision(TransferableObject ad)
	{
		DistanceAdInfo info = (DistanceAdInfo) getLocalInfo(ad);

		if (info.sourceStations.isEmpty())
		{
			info.decision = 0;
		}
		else
		{
			Iterator i = info.sourceStations.iterator();
			double minDistance = ((Station) i.next()).getLocation().getDistanceTo(getComputer().getLocation()); 
			
			while (i.hasNext())
			{
				Station s = (Station) i.next();
				double distance = s.getLocation().getDistanceTo(getComputer().getLocation());
				
				if (distance < minDistance)
				{
					minDistance = distance;
				}
			}
			
			info.decision = minDistance < getDistanceThreshold() ? 0 : 1;
		}
	}

	
	public double getDistanceThreshold()
	{
		return distanceThreshold;
	}

	public void setDistanceThreshold(double d)
	{
		distanceThreshold = d;
	}

	public String getName()
	{
		return "location-based";
	}


	/* (non-Javadoc)
	 * @see org.lucci.madhoc.broadcast.BroadcastingProtocol#isMultipleRebroadcastAllowed()
	 */
	public boolean isMultipleRebroadcastAllowed()
	{
		return false;
	}

}
