package org.lucci.madhoc.broadcasting.malaga;


import java.util.Collection;

import org.lucci.madhoc.broadcast.BroadcastingMonitor;
import org.lucci.madhoc.broadcast.measure.AverageCoverageMeasure;
import org.lucci.madhoc.simulation.Monitor;
import org.lucci.madhoc.simulation.TerminationCondition;
import org.lucci.madhoc.simulation.measure.MeasureHistory;
import org.lucci.madhoc.simulation.measure.Sensor;
import org.lucci.madhoc.simulation.projection.IdentityProjection;
import org.lucci.madhoc.simulation.projection.Projection;
import org.lucci.madhoc.simulation.projection.SquareWindowProjection;
import org.lucci.math.Utilities;

/*
 * Created on Apr 26, 2005
 */

/**
 * @author luc.hogie
 */
public class TerminationConditionMalagena extends TerminationCondition
{
	private double observationTimeInterval = 1;
	private int maxSimulationTimeAllowed = 20;
    
	/* (non-Javadoc)
	 * @see org.lucci.madhoc.simulation.SimulationTerminationCondition#simulationHasCompleted(org.lucci.madhoc.simulation.Simulation)
	 */
	public boolean applicationHasCompleted(Monitor app)
	{
		BroadcastingMonitor broadcastingApp = (BroadcastingMonitor) app;
		double simulatedTime = broadcastingApp.getNetwork().getSimulation().getSimulatedTime();

		if (simulatedTime > getObservationTimeInterval())
		{
            if (simulatedTime > getMaxSimulationTimeAllowed())
            {
                return true;
            }
            else
            {
                int iteration = broadcastingApp.getNetwork().getSimulation().getIteration();
                double startTime = simulatedTime - getObservationTimeInterval();
                double resolution = broadcastingApp.getNetwork().getSimulation().getResolution();
                int startIteration = (int) (startTime / resolution);
                Projection projection = (Projection) app.getNetwork().getProjectionMap().get(SquareWindowProjection.class);
//              Projection projection = (Projection) app.getNetwork().getProjectionMap().get(IdentityProjection.class);
                Sensor averageCoverageSensor = app.getSensorMap().get(AverageCoverageMeasure.class);
                MeasureHistory measures = (MeasureHistory) projection.getMeasureHistoryMap().get(averageCoverageSensor);
                Collection<Double> relevantMeasures = measures.getValues().subList(startIteration, iteration);
//              double sd = Utilities.getStandardDeviation(relevalMeasures);
                double sd = Utilities.getVariationSum(relevantMeasures);
                return sd == 0;
            }
		}
		else
		{
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see org.lucci.madhoc.config.Configurable#configure()
	 */
	public void configure() throws Throwable
	{
		setObservationTimeInterval(getMonitor().getNetwork().getSimulation().getConfiguration().getDouble("termination_condition_malagena_observation_time_interval"));
        setMaxSimulationTimeAllowed(getMonitor().getNetwork().getSimulation().getConfiguration().getInteger("termination_condition_malagena_max_simulation_time_allowed"));
	}

	public double getObservationTimeInterval()
	{
		return observationTimeInterval;
	}
	public void setObservationTimeInterval(double observationTimeInterval)
	{
        if (observationTimeInterval < 0)
            throw new IllegalArgumentException("observation time interval should be > 0");

        this.observationTimeInterval = observationTimeInterval;
	}

    public int getMaxSimulationTimeAllowed()
    {
        return maxSimulationTimeAllowed;
    }

    public void setMaxSimulationTimeAllowed(int maxSimulationTimeAllowed)
    {
        if (maxSimulationTimeAllowed < 0)
            throw new IllegalArgumentException("maxSimulationTimeAllowed should be > 0");

        this.maxSimulationTimeAllowed = maxSimulationTimeAllowed;
    }
    
}
