package org.lucci.madhoc.network.env.mall;



import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Random;
import java.util.Vector;

import org.lucci.config.ConfigurationException;
import org.lucci.madhoc.env.MobilityMedium;
import org.lucci.madhoc.env.MobilityModel;
import org.lucci.madhoc.network.Station;
import org.lucci.madhoc.network.net.Location;
import org.lucci.math.Utilities;
import org.lucci.up.data.Point;



/*
 * Created on Jul 24, 2004
 */

/**
 * @author luc.hogie
 */
public class BohMobility extends MobilityModel
{
    private Spot targetedSpot;
    private List<Spot> visitedSpots = new Vector<Spot>();
    private double acceleration = 1;
    private double inSpotSpeed = 2;
    private double outSpotSpeed = 4;
    private double probabilityToGoToClosestShop;
    private double accelerationFactor;

    
   
    public void setMobileNode(Station station)
    {
        super.setMobileNode(station);
    }




    public Spot getTargetedSpot()
    {
        return targetedSpot;
    }

    public void setTargetedSpot(Spot station)
    {
        targetedSpot = station;
    }

    public boolean isWithinASpot()
    {
        if (getTargetedSpot() == null)
        {
            return false;
        }
        else
        {
            return getMobileNode().getLocation().getDistanceTo(getTargetedSpot().getLocation()) < getTargetedSpot().getRadius();
        }
    }




    /* (non-Javadoc)
     * @see org.lucci.madhoc.Initializable#init(org.lucci.math.relation.Relation)
     */
    public void configure()
        throws ConfigurationException
    {
        Random random = getMobileNode().getNetwork().getSimulation().getRandomNumberGenerator().getRandom();
        setTargetedSpot((Spot) org.lucci.util.Collections.getRandomObject(((HumanEnvironment) getMobileNode().getNetwork().getNetworkEnvironment()).getSpots(), random));
        double randomAngle = Utilities.getRandomBetween(0, 2 * Math.PI, random);
        double randomDistanceFactor = Utilities.getRandomBetween(0, 1, random);
        double x = getTargetedSpot().getLocation().getX() + Math.cos(randomAngle) * getTargetedSpot().getRadius() * randomDistanceFactor; 
        double y = getTargetedSpot().getLocation().getY() + Math.sin(randomAngle) * getTargetedSpot().getRadius() * randomDistanceFactor; 
        getMobileNode().setLocation(new Location(x, y));
        setMobilityMedium(new MobilityMedium.Legs(random));
        getMobilityMedium().setAngle(Utilities.getRandomBetween(0, 2 * Math.PI, random));

        double[] inSpotSpeed = getMobileNode().getNetwork().getSimulation().getConfiguration().getInterval("human_mobility_in_spot_speed");
        setInSpotSpeed(Utilities.getRandomBetween(inSpotSpeed[0], inSpotSpeed[1], random));

        double[] outSpotSpeed = getMobileNode().getNetwork().getSimulation().getConfiguration().getInterval("human_mobility_out_spot_speed");
        setOutSpotSpeed(Utilities.getRandomBetween(outSpotSpeed[0], outSpotSpeed[1], random));
        
        this.probabilityToGoToClosestShop = getMobileNode().getNetwork().getSimulation().getConfiguration().getDouble("probability_to_go_to_closest_shop");
        this.accelerationFactor = getMobileNode().getNetwork().getSimulation().getConfiguration().getDouble("acceleration_factor");
        

    }

    public static void sortSpotsByDistanceToPoint(final Point point, List<Spot> spots)
    {
        Collections.sort(spots, new Comparator()
            {
                public int compare(Object arg0, Object arg1)
                {
                    Spot s0 = (Spot) arg0;
                    Spot s1 = (Spot) arg1;
                    double d0 = s0.getLocation().getDistanceTo(point);
                    double d1 = s1.getLocation().getDistanceTo(point);

                    if (d0 == d1)
                    {
                        return 0;
                    }
                    else if (d0 < d1)
                    {
                        return -1;
                    }
                    else
                    {
                        return 1;
                    }
                }
            });
    }

    /* (non-Javadoc)
     * @see org.lucci.madhoc.mobility.MobilityModel#moveStation(org.lucci.madhoc.network.MobileStation)
     */
    public void moveStation(double duration, double simulatedTime)
    {
        if (getTargetedSpot() == null)
        {
            // find all the spots in the environment
            List<Spot> allSpots = new Vector<Spot>(((HumanEnvironment) getMobileNode().getNetwork().getNetworkEnvironment()).getSpots());

            // process the spots that have not yet been visited
            List<Spot> notYetVisitedShops = new Vector<Spot>(allSpots);
            notYetVisitedShops.removeAll(visitedSpots);

            // if all spots have been visited, state that none
            // of them have actually been visisted, this kinda reinitialize
            // the system
            if (notYetVisitedShops.isEmpty())
            {
                visitedSpots.clear();
                notYetVisitedShops = new Vector<Spot>(allSpots);
            }

            sortSpotsByDistanceToPoint(getMobileNode().getLocation(), notYetVisitedShops);
            
            // some nodes cross the city (go the the farest shop)
            if (getMobileNode().getNetwork().getSimulation().getRandomNumberGenerator().getRandom().nextDouble() > this.probabilityToGoToClosestShop)
            {
                setTargetedSpot(notYetVisitedShops.get(notYetVisitedShops.size() - 1));
            }
            // all of them go to the closest shop
            else
            {
                setTargetedSpot(notYetVisitedShops.get(0));
            }
        }

        boolean wasWithinItsSpot = isWithinASpot();

        double speed = isWithinASpot() ? getInSpotSpeed() : getOutSpotSpeed();
        
        // nodes at the edge are 4 times faster than nodes in the middle
        Point center = new Point(getMobileNode().getNetwork().getNetworkEnvironment().getGrid().getEdgeLenght() / 2, getMobileNode().getNetwork().getNetworkEnvironment().getGrid().getEdgeLenght() / 2);
        double maximumDistanceToCenter = getMobileNode().getNetwork().getNetworkEnvironment().getGrid().getEdgeLenght() * Math.sqrt(2) / 2;
        double distanceToCenter = getMobileNode().getLocation().getDistanceTo(center);
        double acceleration = (this.accelerationFactor - 1) * (distanceToCenter / maximumDistanceToCenter) + 1;  
        speed *= acceleration;

        Station station = getMobileNode();
        double distance = speed * duration;
        double newX = station.getLocation().getX() + distance * Math.cos(getMobilityMedium().getAngle());
        double newY = station.getLocation().getY() + distance * Math.sin(getMobilityMedium().getAngle());

        // if the mobile station goes out of the bounds of the simulation area
        if ( newX < 0 || newX > station.getNetwork().getNetworkEnvironment().getGrid().getEdgeLenght() - 1 || newY < 0 || newY > station.getNetwork().getNetworkEnvironment().getGrid().getEdgeLenght() - 1 )
        {
            // it just randomly changes direction, it can take any direction
            getMobilityMedium().setAngle(Utilities.getRandomBetween(0, 2 * Math.PI, getMobileNode().getNetwork().getSimulation().getRandomNumberGenerator().getRandom()));
        }
        else
        {
            station.getLocation().setXY(newX, newY);

            if (isWithinASpot())
            {
                // slightly changes direction each 1 secondes
                if (simulatedTime % 1000 == 0)
                {
                    getMobilityMedium().setAngle(getMobilityMedium().getAngle() + Utilities.getRandomBetween(-Math.PI / 8, Math.PI / 8, getMobileNode().getNetwork().getSimulation().getRandomNumberGenerator().getRandom()));
                }
            }
            else
            {
                double distanceToShop = getTargetedSpot().getLocation().getDistanceTo(station.getLocation());
                double dx = getTargetedSpot().getLocation().getX() - station.getLocation().getX(); 
                double dy = getTargetedSpot().getLocation().getY() - station.getLocation().getY();
                double cos = dx / distanceToShop;
                double sin = dy / distanceToShop;
                double angleToShop = Math.acos(cos);
                if ( sin < 0 ) angleToShop = 2 * Math.PI - angleToShop;
                getMobilityMedium().setAngle(angleToShop);
            }
        }

        // if the guy just arrives at this shop
        if ( !wasWithinItsSpot && isWithinASpot() )                                     
        {
            getTargetedSpot().getStations().add(getMobileNode());
            visitedSpots.add(getTargetedSpot());
        }
        else if ( wasWithinItsSpot && !isWithinASpot() )
        {
            getTargetedSpot().getStations().remove(this);
            
            // the station will choose a destination spot at the next iteration
            setTargetedSpot(null);
        }
    }

    public double getAcceleration()
    {
        return acceleration;
    }
    public void setAcceleration(double acceleration)
    {
        this.acceleration = acceleration;
    }




    public double getInSpotSpeed()
    {
        return inSpotSpeed;
    }

    public void setInSpotSpeed(double inSpotSpeed)
    {
        this.inSpotSpeed = inSpotSpeed;
    }

    public double getOutSpotSpeed()
    {
        return outSpotSpeed;
    }

    public void setOutSpotSpeed(double outSpotSpeed)
    {
        this.outSpotSpeed = outSpotSpeed;
    }
}
