package org.lucci.madhoc.network.env.mall;

import org.lucci.config.ConfigurationException;
import org.lucci.math.Utilities;
import org.lucci.up.data.Point;

public class CityEnvironment extends HumanEnvironment
{

    @Override
    public Class getDefaultMobilityModel()
    {
        // TODO Auto-generated method stub
        return CityMobility.class;
    }

    protected void locateSpots(int shopCount, double minRadius, double maxRadius)
        throws ConfigurationException
    {
        int celSize = 100;
        double alteration = getNetwork().getSimulation().getConfiguration().getDouble("human_mobility_grid_alteration_ratio") * getNetwork().getNetworkEnvironment().getGrid().getEdgeLenght();
        int rowCount = (int) (getGrid().getEdgeLenght() / celSize);
        
        for (int i = 0; i < rowCount; ++i)
        {
            for (int j = 0; j < rowCount; ++j)
            {
                double x = i * celSize + celSize / 2;
                double y = j * celSize + celSize / 2;
                double proba = getProba(x, y, celSize);

                if (Utilities.getRandomBetween(0, 1, getNetwork().getSimulation().getRandomNumberGenerator().getRandom()) < proba)
                {
                    Spot newSpot = new Spot();
                    newSpot.setRadius(15);
                    x += Utilities.getRandomBetween(-alteration, alteration, getNetwork().getSimulation().getRandomNumberGenerator().getRandom());
                    y += Utilities.getRandomBetween(-alteration, alteration, getNetwork().getSimulation().getRandomNumberGenerator().getRandom());
                    newSpot.setLocation(new Point(x, y));
                    getSpots().add(newSpot);
                }                        
            }
        }
    }

    private double getProba(double x, double y, double celSize)
    {
        Point center = new Point(getGrid().getEdgeLenght() / 2, getGrid().getEdgeLenght() / 2);
        Point point = new Point(x, y);
        double distanceToCenter = point.getDistanceTo(center);
    
        // the point is in the center
        if (distanceToCenter < getGrid().getEdgeLenght() / 4)
        {
            return 0.9;
        }
        // the point is on the peripheral
        else if (getGrid().getEdgeLenght() / 2 - celSize * 0.6 < distanceToCenter && distanceToCenter < getGrid().getEdgeLenght() / 2)
        {
            return 1;
        }
        // the point is on one street heading to the center
        else if ((center.getX() < x && x < center.getX() + celSize) || (center.getY() < y && y < center.getY() + celSize))
        {
            return 1;
        }
        else
        {
            return 0;
        }
    }

}
