package org.lucci.madhoc.network.env.mall;



import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Random;
import java.util.Vector;

import org.lucci.config.ConfigurationException;
import org.lucci.madhoc.env.MobilityMedium;
import org.lucci.madhoc.env.MobilityModel;
import org.lucci.madhoc.network.Station;
import org.lucci.madhoc.network.net.Location;
import org.lucci.math.Utilities;
import org.lucci.up.data.Point;



/*
 * Created on Jul 24, 2004
 */

/**
 * @author luc.hogie
 */
public class HumanMobility extends MobilityModel
{
	private Spot targetedSpot;
	private List<Spot> visitedSpots = new Vector<Spot>();
	private double inSpotSpeed = 2;
	private double outSpotSpeed = 4;

   
	public void setMobileNode(Station station)
	{
		super.setMobileNode(station);
	}




	public Spot getTargetedSpot()
	{
		return targetedSpot;
	}

	public void setTargetedSpot(Spot station)
	{
		targetedSpot = station;
	}




	/* (non-Javadoc)
	 * @see org.lucci.madhoc.Initializable#init(org.lucci.math.relation.Relation)
	 */
	public void configure()
		throws ConfigurationException
	{
		Random random = getMobileNode().getNetwork().getSimulation().getRandomNumberGenerator().getRandom();

        // the target spot is a random one
        setTargetedSpot((Spot) org.lucci.util.Collections.getRandomObject(((HumanEnvironment) getMobileNode().getNetwork().getNetworkEnvironment()).getSpots(), random));
        this.visitedSpots.add(getTargetedSpot());
        getTargetedSpot().getStations().add(getMobileNode());

        // is located randomly within the spot
        double randomAngle = Utilities.getRandomBetween(0, 2 * Math.PI, random);
        double randomDistanceFactor = Utilities.getRandomBetween(0, 1, random);
		double x = getTargetedSpot().getLocation().getX() + Math.cos(randomAngle) * getTargetedSpot().getRadius() * randomDistanceFactor; 
		double y = getTargetedSpot().getLocation().getY() + Math.sin(randomAngle) * getTargetedSpot().getRadius() * randomDistanceFactor; 
		getMobileNode().setLocation(new Location(x, y));

        // the node move on a random angle
        setMobilityMedium(new MobilityMedium.Legs(random));
		getMobilityMedium().setAngle(Utilities.getRandomBetween(0, 2 * Math.PI, random));

        double[] inSpotSpeed = getMobileNode().getNetwork().getSimulation().getConfiguration().getInterval("human_mobility_in_spot_speed");
        setInSpotSpeed(Utilities.getRandomBetween(inSpotSpeed[0], inSpotSpeed[1], random));

        double[] outSpotSpeed = getMobileNode().getNetwork().getSimulation().getConfiguration().getInterval("human_mobility_out_spot_speed");
        setOutSpotSpeed(Utilities.getRandomBetween(outSpotSpeed[0], outSpotSpeed[1], random));
	}

	private static void sortSpotsByDistanceToPoint(final Point point, List<Spot> spots)
	{
		Collections.sort(spots, new Comparator<Spot>()
			{
				public int compare(Spot s0, Spot s1)
				{
					double d0 = s0.getLocation().getDistanceTo(point);
					double d1 = s1.getLocation().getDistanceTo(point);

					if (d0 == d1)
					{
						return 0;
					}
					else if (d0 < d1)
					{
						return -1;
					}
					else
					{
						return 1;
					}
				}
			});
	}

    private Spot findTheClosestSpotTo(Point point, Collection<Spot> spots)
    {
        Spot closestSpot = null;
        double distanceToClosestSpot = Double.MAX_VALUE;
        
        for (Spot spot : spots)
        {
            double distanceToSpot = spot.getLocation().getDistanceTo(point);
            
            if (distanceToSpot < distanceToClosestSpot)
            {
                distanceToClosestSpot = distanceToSpot;
                closestSpot = spot;
            }
        }
        
        return closestSpot;
    }
    
    private Collection<Spot> findNonVisitedSpots()
    {
        List<Spot> allSpots = new Vector<Spot>(((HumanEnvironment) getMobileNode().getNetwork().getNetworkEnvironment()).getSpots());
        List<Spot> notYetVisitedSpots = new Vector<Spot>(allSpots);
        notYetVisitedSpots.removeAll(visitedSpots);
        return notYetVisitedSpots;
    }

    
	/* (non-Javadoc)
	 * @see org.lucci.madhoc.mobility.MobilityModel#moveStation(org.lucci.madhoc.network.MobileStation)
	 */
	public void moveStation(double duration, double simulatedTime)
	{
        // if the node doesn't have any target yet, it needs to find one
		if (getTargetedSpot() == null)
		{
            Collection<Spot> notYetVisitedSpots = findNonVisitedSpots();

            // if all spots have been visited, state that none
            // of them have actually been visisted, this kinda reinitialize
            // the system
			if (notYetVisitedSpots.isEmpty())
			{
				this.visitedSpots.clear();
				notYetVisitedSpots = new Vector<Spot>(((HumanEnvironment) getMobileNode().getNetwork().getNetworkEnvironment()).getSpots());
			}

			Spot closestSpot = findTheClosestSpotTo(getMobileNode().getLocation(), notYetVisitedSpots);
            setTargetedSpot(closestSpot);

            // the distance to the spot has changed since the node has moved
            double distanceToSpot = getTargetedSpot().getLocation().getDistanceTo(getMobileNode().getLocation());

            // so we adjust angle so that the station precisely heads to the station
            double dx = getTargetedSpot().getLocation().getX() - getMobileNode().getLocation().getX(); 
            double dy = getTargetedSpot().getLocation().getY() - getMobileNode().getLocation().getY();
            double cos = dx / distanceToSpot;
            double sin = dy / distanceToSpot;
            double angleToShop = Math.acos(cos);
            if (sin < 0) angleToShop = 2 * Math.PI - angleToShop;
            getMobilityMedium().setAngle(angleToShop);
		}



        
		boolean wasWithinItsSpot = getTargetedSpot().include(getMobileNode().getLocation());
		double speed = wasWithinItsSpot ? getInSpotSpeed() : getOutSpotSpeed();
		double distanceToGo = speed * duration;
        double distanceToSpot = getTargetedSpot().getLocation().getDistanceTo(getMobileNode().getLocation());

        // if the node jump a distance greater that the distance to the targeted spot
        // we assume that it directly go in the middle of the spot
        if (!wasWithinItsSpot)
        {
            if (distanceToGo > distanceToSpot)
            {
                // locate the node in the spot, but not at the center
                distanceToGo = distanceToSpot;
            }
        }

        double newX = getMobileNode().getLocation().getX() + distanceToGo * Math.cos(getMobilityMedium().getAngle());
        double newY = getMobileNode().getLocation().getY() + distanceToGo * Math.sin(getMobilityMedium().getAngle());
        getMobileNode().getLocation().setXY(newX, newY);
        

        // if the station has reached its target spot
        if (getTargetedSpot().include(getMobileNode().getLocation()))
        {
            // by roaming inside, it
            // slightly changes direction each second
            if (((int) simulatedTime) == simulatedTime)
            {
                getMobilityMedium().setAngle(getMobilityMedium().getAngle() + Utilities.getRandomBetween(-Math.PI / 2, Math.PI / 2, getMobileNode().getNetwork().getSimulation().getRandomNumberGenerator().getRandom()));
            }
        }
        // it has not reached its target spot
        else
        {
        }

		// if the guy just arrives at this spot
		if ( !wasWithinItsSpot && getTargetedSpot().include(getMobileNode().getLocation()) )										
		{
			getTargetedSpot().getStations().add(getMobileNode());
			visitedSpots.add(getTargetedSpot());
		}
        // it just left the spot
		else if ( wasWithinItsSpot && !getTargetedSpot().include(getMobileNode().getLocation()) )
		{
			getTargetedSpot().getStations().remove(this);
			
			// the station will choose a destination spot at the next iteration
			setTargetedSpot(null);
		}
	}



	public double getInSpotSpeed()
    {
        return inSpotSpeed;
    }

    public void setInSpotSpeed(double inSpotSpeed)
    {
        this.inSpotSpeed = inSpotSpeed;
    }

    public double getOutSpotSpeed()
    {
        return outSpotSpeed;
    }

    public void setOutSpotSpeed(double outSpotSpeed)
    {
        this.outSpotSpeed = outSpotSpeed;
    }
}
