package org.lucci.madhoc.network.env.mall;



import org.lucci.madhoc.env.RadioPropagationModel;
import org.lucci.madhoc.network.Station;
import org.lucci.madhoc.network.Connection;

/*
 * Created on Jul 25, 2004
 */

/**
 * @author luc.hogie
 */
public class HumanRadioPropagationModel extends RadioPropagationModel
{
	private double wallObstruction = 0.8;
	
	/* (non-Javadoc)
	 * @see org.lucci.madhoc.network.propagation.WavePropagationModel#applyEnvironmentalConstraints(org.lucci.madhoc.network.Connection)
	 */
	public double getEnvironmentPerturbation(Connection connection)
	{
		Station s1 = connection.getNetworkInterface1().getNetworkingUnit().getStation();
		Station s2 = connection.getNetworkInterface2().getNetworkingUnit().getStation();
		HumanMobility m1 = (HumanMobility) s1.getMobilityModel();
		HumanMobility m2 = (HumanMobility) s2.getMobilityModel();

        if (m1.getTargetedSpot() == null || m2.getTargetedSpot() == null)
        {
            return 0;
        }
		// if both stations are inside the shop
		if (m1.getTargetedSpot().include(m1.getMobileNode().getLocation())
                   && m2.getTargetedSpot().include(m2.getMobileNode().getLocation()))
		{
			if (m1.getTargetedSpot() == m2.getTargetedSpot())
			{
				// the stations are in the same shop
				// people moving around and shops perturbates the signal
				return 0;
			}
			else
			{
				// they are not in the same shop
				// the signal needs to cross 2 walls
				return 1 - Math.pow(1 - wallObstruction, 2);
			}
		}
		// if both stations are outside
		else if (!m1.getTargetedSpot().include(m1.getMobileNode().getLocation())
                && !m2.getTargetedSpot().include(m2.getMobileNode().getLocation()))
		{
			// no walls, nothing, no perturbation
			return 0;
		}
		else
		{
			// hotspots walls stop most of the signal
			return wallObstruction;
		}
	}

	/* (non-Javadoc)
	 * @see org.lucci.madhoc.config.Configurable#configure()
	 */
	public void configure() throws Throwable
	{
		setWallObstruction(getNetworkEnvironment().getNetwork().getSimulation().getConfiguration().getDouble("human_environment_wall_obstruction"));
	}

	public double getWallObstruction()
	{
		return wallObstruction;
	}
	
	public void setWallObstruction(double wallObstruction)
	{
		if (wallObstruction < 0 || wallObstruction > 1)
			throw new IllegalArgumentException("wall obstruction must be in [0, 1]");

		this.wallObstruction = wallObstruction;
	}
}
