package org.lucci.madhoc.network.env.open_area;


import org.lucci.config.ConfigurationException;
import org.lucci.madhoc.env.MobilityModel;
import org.lucci.madhoc.network.net.Location;
import org.lucci.math.Utilities;
import org.lucci.up.data.Point;

/*
 * Created on Jul 24, 2004
 */

/**
 * @author luc.hogie
 */
public class RandomWaypointMobility extends MobilityModel
{
	private double pausetimeLeft = -1;
	private Point destination;
	private double minimumPause = 1;
	private double maximumPause = 5;
	private double minimumVelocity = 2/3.6;
	private double maximumVelocity = 6/3.6;

	public double getMaximumPause()
	{
		return maximumPause;
	}
	public void setMaximumPause(double maxiumPause)
	{
		this.maximumPause = maxiumPause;
	}
	public double getMinimumPause()
	{
		return minimumPause;
	}
	public void setMinimumPause(double minimumPause)
	{
		this.minimumPause = minimumPause;
	}

	double previousDistance = Double.MAX_VALUE;
	
	/* (non-Javadoc)
	 * @see org.lucci.madhoc.mobility.MobilityModel#moveStation(org.lucci.madhoc.network.MobileStation, double, long)
	 */
	public void moveStation(double duration, double simulatedTime)
	{
		// the node is not having a pause
		if (pausetimeLeft == -1)
		{
			if (getDestination() == null)
			{
				destination = getRandomPoint();
				double distanceToNewDestination = destination.getDistanceTo(getMobileNode().getLocation());
				previousDistance = distanceToNewDestination;
				double dx = destination.getX() - getMobileNode().getLocation().getX();
				double dy = destination.getY() - getMobileNode().getLocation().getY();
				double cos = dx / distanceToNewDestination;
				double sin = dy / distanceToNewDestination;
				double angle = Utilities.getAngle(cos, sin);
				getMobilityMedium().setAngle(angle);
				getMobilityMedium().setSpeed(Utilities.getRandomBetween(getMinimumVelocity(), getMaximumVelocity(), getMobileNode().getNetwork().getSimulation().getRandomNumberGenerator().getRandom()));
				moveStation(duration, simulatedTime);
			}
			else
			{
				double speed = getMobilityMedium().getSpeed();
				double distance = speed * duration;
				double angle = getMobilityMedium().getAngle();
				double newX = getMobileNode().getLocation().getX() + distance * Math.cos(angle);
				double newY = getMobileNode().getLocation().getY() + distance * Math.sin(angle);
				double distanceToDestination = destination.getDistanceTo(new Location(newX, newY));

				// if the node has reached its destination
				if (previousDistance > 0 &&
					(distanceToDestination > previousDistance || newX < 0 || newY < 0
						|| newX > getMobileNode().getNetwork().getNetworkEnvironment().getGrid().getEdgeLenght() - 1
						|| newY > getMobileNode().getNetwork().getNetworkEnvironment().getGrid().getEdgeLenght() - 1))
				{
					pausetimeLeft = Utilities.getRandomBetween(getMinimumPause(), getMaximumPause(), getMobileNode().getNetwork().getSimulation().getRandomNumberGenerator().getRandom());
					getMobileNode().getLocation().setXY(destination.getX(), destination.getY());
					destination = null;
					previousDistance = -1;
				}
				else
				{
					previousDistance = distanceToDestination;
					getMobileNode().getLocation().setXY(newX, newY);
				}
			}
		}
		else
		{
			if (pausetimeLeft > 0)
			{
				pausetimeLeft -= duration;
			}
			else
			{
				pausetimeLeft = -1;
				moveStation(duration, simulatedTime);
			}
		}
	}


	private Point getRandomPoint()
	{
		double edgeSize = getMobileNode().getNetwork().getNetworkEnvironment().getGrid().getEdgeLenght();
		double x = Utilities.getRandomBetween(0, edgeSize, getMobileNode().getNetwork().getSimulation().getRandomNumberGenerator().getRandom()); 
		double y = Utilities.getRandomBetween(0, edgeSize, getMobileNode().getNetwork().getSimulation().getRandomNumberGenerator().getRandom());
		return new Point(x, y);
	}

	public Point getDestination()
	{
		return destination;
	}
	public double getMaximumVelocity()
	{
		return maximumVelocity;
	}
	public void setMaximumVelocity(double maxiumVelocity)
	{
		this.maximumVelocity = maxiumVelocity;
	}
	public double getMinimumVelocity()
	{
		return minimumVelocity;
	}
	public void setMinimumVelocity(double minimumVelocity)
	{
		this.minimumVelocity = minimumVelocity;
	}
	
	/* (non-Javadoc)
	 * @see org.lucci.madhoc.Initializable#init(org.lucci.math.relation.Relation)
	 */
	public void configure() throws ConfigurationException
	{
		double[] pauseInterval = getMobileNode().getNetwork().getSimulation().getConfiguration().getInterval("random_waypoint_mobility_pause_interval");
		setMinimumPause(pauseInterval[0]);
		setMaximumPause(pauseInterval[1]);

		double[] velocityInterval = getMobileNode().getNetwork().getSimulation().getConfiguration().getInterval("random_waypoint_mobility_velocity_interval");
		setMinimumVelocity(velocityInterval[0]);
		setMaximumVelocity(velocityInterval[1]);

		pausetimeLeft = -1;

		Point randomPoint = getRandomPoint();
		getMobileNode().setLocation(new Location(randomPoint.getX(), randomPoint.getY()));
	}

	public double getPausetimeLeft()
	{
		return pausetimeLeft;
	}
	public void setPausetimeLeft(double pausetimeLeft)
	{
		this.pausetimeLeft = pausetimeLeft;
	}
}
